# -*- coding: utf-8 -*-

import pytest
import sys
import cPickle as pickle
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

from aerospike import predicates as p

class TestApply(TestBaseClass):
    def setup_class(cls):
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            TestApply.client = aerospike.client(config).connect()
        else:
            TestApply.client = aerospike.client(config).connect(user, password)
        TestApply.skip_old_server = True
        versioninfo = TestApply.client.info('version')
        for keys in versioninfo:
            for value in versioninfo[keys]:
                if value != None:
                    versionlist = value[value.find("build") + 6:value.find("\n")].split(".")
                    if int(versionlist[0]) >= 3 and int(versionlist[1]) >= 6:
                        TestApply.skip_old_server = False

        policy = {}
        TestApply.client.index_integer_create('test', 'demo', 'age',
                                              'age_index', policy)
        policy = {}
        TestApply.client.index_integer_create('test', 'demo', 'age1',
                                              'age_index1', policy)

        policy = {}
        filename = "sample.lua"
        udf_type = 0

        status = TestApply.client.udf_put(filename, udf_type, policy)
        filename = "test_record_udf.lua"
        status = TestApply.client.udf_put(filename, udf_type, policy)
        filename = "udf_basic_ops.lua"
        status = TestApply.client.udf_put(filename, udf_type, policy)

    def teardown_class(cls):
        policy = {}
        TestApply.client.index_remove('test', 'age_index', policy)
        TestApply.client.index_remove('test', 'age_index1', policy)
        policy = {'timeout': 0}
        module = "sample.lua"

        status = TestApply.client.udf_remove(module, policy)
        module = "udf_basic_ops.lua"
        status = TestApply.client.udf_remove(module, policy)
        TestApply.client.close()

    def setup_method(self, method):
        """
        Setup method.
        """

        for i in xrange(5):
            key = ('test', 'demo', i)
            rec = {
                'name': ['name%s' % (str(i))],
                'addr': 'name%s' % (str(i)),
                'age': i,
                'no': i,
                'basic_map': {"k30": 6,
                              "k20": 5,
                              "k10": 1}
            }
            TestApply.client.put(key, rec)

    def teardown_method(self, method):
        """
        Teardown method.
        """
        for i in xrange(5):
            key = ('test', 'demo', i)
            TestApply.client.remove(key)

    def test_apply_with_no_paramters(self):
        """
            Invoke apply() without any mandatory parameters.
        """
        with pytest.raises(TypeError) as typeError:
            TestApply.client.apply()

        assert "Required argument 'key' (pos 1) not found" in typeError.value

    def test_apply_with_correct_parameters(self):
        """
            Invoke apply() with correct arguments
        """
        key = ('test', 'demo', 1)
        retval = TestApply.client.apply(key, 'sample', 'list_append', ['name',
                                                                       'car'])
        (key, meta, bins) = TestApply.client.get(key)

        assert bins['name'] == ['name1', 'car']
        assert retval == 0

    def test_apply_with_correct_parameters_float_argument(self):
        """
            Invoke apply() with correct arguments with a floating value in the
            list of arguments
        """
        if TestApply.skip_old_server == True:
            pytest.skip("Server does not support apply on float type as lua argument")
        key = ('test', 'demo', 1)
        retval = TestApply.client.apply(key, 'sample', 'list_append', ['name',
                                                                       5.434])
        (key, meta, bins) = TestApply.client.get(key)

        assert bins['name'] == ['name1', 5.434]
        assert retval == 0

    def test_apply_with_none_parameter(self):
        """
            Invoke apply() with a None argument
        """
        key = ('test', 'demo', 1)
        retval = TestApply.client.apply(key, 'sample', 'list_append', ['name',
                                                                       None])
        (key, meta, bins) = TestApply.client.get(key)

        assert bins['name'] == ['name1', None]
        assert retval == 0

    def test_apply_with_policy(self):
        """
            Invoke apply() with policy
        """
        policy = {'timeout': 1000}
        key = ('test', 'demo', 1)
        retval = TestApply.client.apply(key, 'sample', 'list_append',
                                        ['name', 'car'], policy)
        (key, meta, bins) = TestApply.client.get(key)
        assert retval == 0
        assert bins['name'] == ['name1', 'car']

    def test_apply_with_incorrect_policy(self):
        """
            Invoke apply() with incorrect policy
        """
        policy = {'timeout': 0.1}
        key = ('test', 'demo', 1)
        try:
            retval = TestApply.client.apply(key, 'sample', 'list_append', ['name',
'car'], policy)

        except ParamError as exception:
            assert exception.code == -2L
            assert exception.msg == 'timeout is invalid'

    def test_apply_with_extra_argument(self):
        """
            Invoke apply() with extra argument
        """
        policy = {'timeout': 1000}
        key = ('test', 'demo', 1)
        with pytest.raises(TypeError) as typeError:
            TestApply.client.apply(key, 'sample', 'list_append',
                                   ['name', 'car'], policy, "")

        assert "apply() takes at most 5 arguments (6 given)" in typeError.value

    def test_apply_with_incorrect_bin(self):
        """
            Invoke apply() with incorrect bin
        """
        policy = {'timeout': 1000}
        key = ('test', 'demo', 1)
        try:
            retval = TestApply.client.apply(key, 'sample', 'list_append', ['addr',
'car'], policy)

        except UDFError as exception:
            assert exception.code == 100L

    def test_apply_with_empty_module_function(self):
        """
            Invoke apply() with empty module and function
        """
        try:
            key = ('test', 'demo', 1)
            retval = TestApply.client.apply(key, '', '', ['name', 'car'])

        except UDFError as exception:
            assert exception.code == 100L
            assert exception.msg == 'UDF: Execution Error 1'

    def test_apply_with_incorrect_module(self):
        """
            Invoke apply() with incorrect module
        """
        try:
            key = ('test', 'demo', 1)
            TestApply.client.apply(key, 'samplewrong', 'list_append', ['name',
                                                                       'car'])

        except UDFError as exception:
            assert exception.code == 100L
            assert exception.msg == 'UDF: Execution Error 1'

    def test_apply_with_incorrect_function(self):
        """
            Invoke apply() with incorrect function
        """
        try:
            key = ('test', 'demo', 1)
            TestApply.client.apply(key, 'sample', 'list_prepend', ['name',
                                                                   'car'])

        except UDFError as exception:
            assert exception.code == 100L
            assert exception.msg == 'function not found'

    def test_apply_with_key_as_string(self):
        """
            Invoke apply() with key as string
        """
        try:
            TestApply.client.apply("", 'sample', 'list_append', ['name', 'car'])

        except ParamError as exception:
            assert exception.code == -2L
            assert exception.msg == 'key is invalid'

    def test_apply_with_incorrect_ns_set(self):
        """
            Invoke apply() with incorrect ns and set
        """
        try:
            key = ('test1', 'demo1', 1)
            TestApply.client.apply(key, 'sample', 'list_prepend', ['name',
                                                                   'car'])

        except NamespaceNotFound as exception:
            assert exception.code == 20L
            assert exception.msg == 'AEROSPIKE_ERR_NAMESPACE_NOT_FOUND'

    def test_apply_with_key_as_none(self):
        """
            Invoke apply() with key as none
        """
        try:
            TestApply.client.apply(None, 'sample', 'list_append', ['name', 'car'])

        except ParamError as exception:
            assert exception.code == -2L
            assert exception.msg == 'key is invalid'

    def test_apply_with_append_integer(self):
        """
            Invoke apply() with append an integer
        """
        key = ('test', 'demo', 1)
        retval = TestApply.client.apply(key, 'sample', 'list_append', ['name',
                                                                       1])
        assert retval == 0
        (key, meta, bins) = TestApply.client.get(key)
        assert bins['name'] == ['name1', 1]

    def test_apply_with_append_list(self):
        """
            Invoke apply() with append an list
        """
        key = ('test', 'demo', 1)
        retval = TestApply.client.apply(key, 'sample', 'list_append', ['name',
                                                                       [1, 2]])
        assert retval == 0
        (key, meta, bins) = TestApply.client.get(key)
        assert bins['name'] == ['name1', [1, 2]]

    def test_apply_with_integer(self):
        """
            Invoke apply() with integer
        """
        key = ('test', 'demo', 1)
        retval = TestApply.client.apply(key, 'test_record_udf',
                                        'bin_udf_operation_integer', ['age', 2,
                                                                      20])
        assert retval == 23
        (key, meta, bins) = TestApply.client.get(key)
        assert bins['age'] == 23

    def test_apply_with_string(self):
        """
            Invoke apply() with string
        """
        key = ('test', 'demo', 1)
        retval = TestApply.client.apply(key, 'test_record_udf',
                                        'bin_udf_operation_string', ['addr',
                                                                     " world"])
        assert retval == "name1 world"
        (key, meta, bins) = TestApply.client.get(key)
        assert bins['addr'] == "name1 world"

    def test_apply_with_map(self):
        """
            Invoke apply() with map
        """
        key = ('test', 'demo', 1)
        retval = TestApply.client.apply(key, 'test_record_udf', 'map_iterate',
                                        ['basic_map', 555])
        assert retval == None
        (key, meta, bins) = TestApply.client.get(key)
        assert bins['basic_map'] == {"k30": 555, "k20": 555, "k10": 555}

    def test_apply_with_record(self):
        """
            Invoke apply() with record
        """
        key = ('test', 'demo', 1)
        retval = TestApply.client.apply(key, 'test_record_udf',
                                        'udf_returns_record', [])
        assert retval != None

    def test_apply_with_bytearray(self):
        """
            Invoke apply() with a bytearray as a argument
        """
        key = ('test', 'demo', 'apply_insert')
        retval = TestApply.client.apply(key, 'udf_basic_ops',
                                        'create_record',
                                        [bytearray("asd;as[d'as;d",
                                            "utf-8")])
        (key, meta, bins) = TestApply.client.get(key)

        assert bins == {'bin': bytearray(b"asd;as[d\'as;d")}

        TestApply.client.remove(key)

    def test_apply_with_extra_argument_to_lua(self):
        """
            Invoke apply() with extra argument to lua
        """
        key = ('test', 'demo', 1)
        retval = TestApply.client.apply(key, 'sample', 'list_append',
                                        ['name', 'car', 1])
        assert retval == 0
        (key, meta, bins) = TestApply.client.get(key)
        assert bins['name'] == ['name1', 'car']

    def test_apply_with_extra_argument_in_lua(self):
        """
            Invoke apply() with extra argument in lua
        """
        key = ('test', 'demo', 1)
        retval = TestApply.client.apply(key, 'sample', 'list_append_extra',
                                        ['name', 'car'])
        assert retval == 0
        (key, meta, bins) = TestApply.client.get(key)
        assert bins['name'] == ['name1', 'car']

    def test_apply_with_no_argument_in_lua(self):
        """
            Invoke apply() with no argument in lua
        """
        key = ('test', 'demo', 1)
        with pytest.raises(TypeError) as typeError:
            retval = TestApply.client.apply(key, 'sample', 'list_append_extra')
        assert "Required argument 'args' (pos 4) not found" in typeError.value

    def test_apply_with_append_list(self):
        """
            Invoke apply() with append an list
        """
        key = ('test', 'demo', 1)
        retval = TestApply.client.apply(key, 'sample', 'list_append', ['name',
                                                                       [1, 2]])
        assert retval == 0
        (key, meta, bins) = TestApply.client.get(key)
        assert bins['name'] == ['name1', [1, 2]]

    def test_apply_with_unicode_module_and_function(self):
        """
            Invoke apply() with unicode module and function
        """
        key = ('test', 'demo', 1)
        retval = TestApply.client.apply(key, u'sample', u'list_append',
                                        ['name', 'car'])
        (key, meta, bins) = TestApply.client.get(key)

        assert bins['name'] == ['name1', 'car']
        assert retval == 0

    def test_apply_with_correct_parameters_without_connection(self):
        """
            Invoke apply() with correct arguments without connection
        """
        key = ('test', 'demo', 1)
        config = {'hosts': [('127.0.0.1', 3000)]}
        client1 = aerospike.client(config)
        try:
            retval = client1.apply(key, 'sample', 'list_append', ['name', 'car'])

        except ClusterError as exception:
            assert exception.code == 11L
            assert exception.msg == 'No connection to aerospike cluster'



# -*- coding: utf-8 -*-

import pytest
import sys
import time
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestCreateUser(TestBaseClass):

    pytestmark = pytest.mark.skipif(
        TestBaseClass().get_hosts()[1] == None,
        reason="No user specified, may be not secured cluster.")

    def setup_method(self, method):
        """
        Setup method
        """
        hostlist, user, password = TestBaseClass().get_hosts()
        config = {"hosts": hostlist}

        self.client = aerospike.client(config).connect(user, password)

        self.delete_users = []

    def teardown_method(self, method):
        """
        Teardown method
        """

        policy = {}

        for user in self.delete_users:
            self.client.admin_drop_user( user, policy )

        self.client.close()

    def test_create_user_without_any_parameters(self):

        with pytest.raises(TypeError) as typeError:
            self.client.admin_create_user()

        assert "Required argument 'user' (pos 1) not found" in typeError.value

    def test_create_user_with_proper_parameters(self):

        policy = {"timeout": 1000}
        user = "user1-test"
        password = "user1-test"
        roles = ["read", "read-write", "sys-admin"]

        try:
            self.client.admin_drop_user ( user, policy )
        except:
            pass

        status = self.client.admin_create_user( user, password, roles, policy )

        time.sleep(2)

        assert status == 0

        user_details = self.client.admin_query_user( user, policy )

        assert user_details == ['read', 'read-write', 'sys-admin']

        self.delete_users.append('user1-test')

    def test_create_user_with_proper_parameters_without_policy(self):

        policy = { "timeout": 1000 }
        user = "user1-test"
        password = "user1-test"
        roles = ["read", "read-write", "sys-admin"]

        try:
            self.client.admin_drop_user ( user, policy )
        except:
            pass

        status = self.client.admin_create_user( user, password, roles )

        time.sleep(2)

        assert status == 0

        user_details = self.client.admin_query_user( user, policy )

        assert user_details == ['read', 'read-write', 'sys-admin']

        self.delete_users.append('user1-test')

    def test_create_user_with_invalid_timeout_policy_value(self):

        policy = {"timeout": 0.1}
        user = "user3-test"
        password = "user3-test"
        roles = ['sys-admin']

        try:
            self.client.admin_drop_user ( user, policy )
        except:
            pass

        try:
            status = self.client.admin_create_user( user, password, roles, policy )

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "timeout is invalid"

    def test_create_user_with_proper_timeout_policy_value(self):

        policy = {'timeout': 20}
        user = "user2-test"
        password = "user2-test"
        roles = ["read-write", "sys-admin"]

        try:
            self.client.admin_drop_user ( user, policy )
        except:
            pass

        status = self.client.admin_create_user( user, password, roles , policy )

        time.sleep(2)

        assert status == 0

        user_details = self.client.admin_query_user( user )

        assert user_details == ['read-write', 'sys-admin']

        self.delete_users.append('user2-test')

    def test_create_user_with_none_username(self):

        policy = {'timeout': 20}
        user = None
        password = "user3-test"
        roles = ["sys-admin"]

        try:
            status = self.client.admin_create_user( user, password, roles, policy )

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Username should be a string"

    def test_create_user_with_empty_username(self):

        policy = {}
        user = ""
        password = "user3-test"
        roles = ["read-write"]

        try:
            status = self.client.admin_create_user( user, password, roles, policy )

        except InvalidUser as exception:
            assert exception.code == 60
            assert exception.msg == "AEROSPIKE_INVALID_USER"

    def test_create_user_with_special_characters_in_username(self):

        policy = {}
        user = "!#Q#AEQ@#$%&^*((^&*~~~````"
        password = "uesr4-test"
        roles = ["read-write"]

        try:
            self.client.admin_drop_user ( user, policy )
        except:
            pass

        status = self.client.admin_create_user( user, password, roles, policy )

        assert status == 0

        self.delete_users.append(user)

    def test_create_user_with_none_password(self):

        policy = {}
        user = "uesr4-test"
        password = None
        roles = ["sys-admin"]

        try:
            status = self.client.admin_create_user( user, password, roles, policy )

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Password should be a string"

    def test_create_user_with_empty_string_as_password(self):

        policy = {}
        user = "user5-test"
        password = ""
        roles = ["read-write"]

        try:
            self.client.admin_drop_user ( user, policy )
        except:
            pass

        status = self.client.admin_create_user( user, password, roles, policy )

        assert status == 0
        time.sleep(2)
        self.delete_users.append(user)

    def test_create_user_with_special_characters_in_password(self):

        policy = {}
        user = "user6-test"
        password = "@#!$#$WERWE%&%$"
        roles = ["sys-admin"]

        try:
            self.client.admin_drop_user ( user, policy )
        except:
            pass

        status = self.client.admin_create_user( user, password, roles, policy )

        assert status == 0

        self.delete_users.append(user)

    def test_create_user_with_too_long_username(self):

        policy = {}
        user = "user$" * 1000
        password = "user10-test"
        roles = ["sys-admin"]

        try:
            self.client.admin_drop_user ( user, policy )
        except:
            pass

        try:
            status = self.client.admin_create_user( user, password, roles, policy )

        except InvalidUser as exception:
            assert exception.code == 60
            assert exception.msg == "AEROSPIKE_INVALID_USER"

    def test_create_user_with_too_long_password(self):

        policy = {'timeout': 1000}
        user = "user10-test"
        password = "user#" * 1000
        roles = ["read-write"]

        try:
            self.client.admin_drop_user ( user, policy )
        except:
            pass

        status = self.client.admin_create_user( user, password, roles, policy )

        assert status == 0
        time.sleep(1)

        user_details = self.client.admin_query_user( user, policy )

        assert user_details == ['read-write']

        self.delete_users.append(user)

    def test_create_user_with_empty_roles_list(self):

        policy = {}
        user = "user7"
        password = "user7"
        roles = []

        try:
            self.client.admin_drop_user ( user, policy )
        except:
            pass

        try:
            status = self.client.admin_create_user( user, password, roles, policy )

        except InvalidRole as exception:
            assert exception.code == 70
            assert exception.msg == "AEROSPIKE_INVALID_ROLE"

    def test_create_user_with_non_user_admin_user(self):

        policy = {}
        user = "non_admin_test"
        password = "non_admin_test"
        roles = ["read-write"]

        try:
            self.client.admin_drop_user( user, policy )
        except:
            pass

        status = self.client.admin_create_user( user, password, roles, policy )

        assert status == 0

        config = {"hosts": TestCreateUser.hostlist}

        non_admin_client = None

        try:
            non_admin_client = aerospike.client(config).connect( "non_admin_test", "non_admin_test" )
            status = non_admin_client.admin_create_user( "user78", password, roles, policy )

            if non_admin_client:
                non_admin_client.close()

        except RoleViolation as exception:
            assert exception.code == 81

        self.delete_users.append("non_admin_test")



# -*- coding: utf-8 -*-

import pytest
import sys
import time
import marshal
import json
import pickle
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)


class SomeClass(object):

    pass

class TestPythonSerializer(object):

    def setup_class(cls):
        """
            Setup class
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            TestPythonSerializer.client = aerospike.client(config).connect()
        else:
            TestPythonSerializer.client = aerospike.client(config).connect(user, password)

        # Unset previously set class serializers if set
        aerospike.unset_serializers()

    def teardown_class(cls):
        TestPythonSerializer.client.close()

    def setup_method(self, method):
        """
            Setup method
        """

        self.delete_keys = []

    def teardown_method(self, method):
        """
            Teardown method
        """
        for key in self.delete_keys:
            try:
                TestPythonSerializer.client.remove(key)
            except:
                pass


    def test_instance_serializer_and_no_class_serializer(self):

        #    Invoke put() for record with no class serializer. There is an
        #    instance serializer
        hostlist, user, password = TestBaseClass.get_hosts()
        method_config = {
            'hosts': hostlist,
            'serialization': ((lambda v: json.dumps(v)), (lambda v: json.loads(v)))}
        if user == None and password == None:
            client = aerospike.client(method_config).connect()
        else:
            client = aerospike.client(method_config).connect(user, password)
        key = ('test', 'demo', 11)
        try:
            client.remove(key)
        except:
            pass

        rec = {'normal': 1234, 'tuple': (1,2,3)}

        res = client.put(key, rec, serializer=aerospike.SERIALIZER_USER)

        assert res == 0

        _, _, bins = client.get(key)

        # tuples JSON-encode to a list, and we use this fact to check which
        # serializer ran:
        assert bins == {'normal': 1234, 'tuple': [1,2,3]}
        client.remove(key)
        client.close()


    def test_builtin_with_instance_serializer_and_no_class_serializer(self):

        #    Invoke put() for data record with builtin serializer and an
        #    instance serializer set
        hostlist, user, password = TestBaseClass.get_hosts()
        method_config = {
            'hosts': hostlist,
            'serialization': ((lambda v: json.dumps(v)), (lambda v: json.loads(v)))}
        if user == None and password == None:
            client = aerospike.client(method_config).connect()
        else:
            client = aerospike.client(method_config).connect(user, password)
        key = ('test', 'demo', 12)
        try:
            client.remove(key)
        except:
            pass

        rec = {'normal': 1234, 'tuple': (1,2,3)}

        res = client.put(key, rec, serializer=aerospike.SERIALIZER_PYTHON)

        assert res == 0

        _, _, bins = client.get(key)

        assert bins == {'normal': 1234, 'tuple': (1,2,3)}
        client.remove(key)
        client.close()


    def test_builtin_with_class_serializer(self):
        """
            Invoke put() for mixed data record with builtin serializer
        """
        aerospike.set_serializer((lambda v: json.dumps(v)))
        aerospike.set_deserializer((lambda v: json.loads(v)))
        key = ('test', 'demo', 13)
        try:
            TestPythonSerializer.client.remove(key)
        except:
            pass

        rec = {'normal': 1234, 'tuple': (1,2,3)}
        res = TestPythonSerializer.client.put(key, rec, serializer=aerospike.SERIALIZER_PYTHON)

        assert res == 0

        _, _, bins = TestPythonSerializer.client.get(key)

        # tuples JSON-encode to a list, and we use this fact to check which
        # serializer ran:
        assert bins == {'normal': 1234, 'tuple': (1,2,3)}
        self.delete_keys.append(key)


    def test_with_class_serializer(self):
        """
            Invoke put() for mixed data record with class serializer.
        """
        aerospike.set_serializer((lambda v: json.dumps(v)))
        aerospike.set_deserializer((lambda v: json.loads(v)))
        key = ('test', 'demo', 14)
        try:
            TestPythonSerializer.client.remove(key)
        except:
            pass

        rec = {'normal': 1234, 'tuple': (1,2,3)}
        res = TestPythonSerializer.client.put(key, rec, serializer=aerospike.SERIALIZER_USER)

        assert res == 0

        _, _, bins = TestPythonSerializer.client.get(key)

        # tuples JSON-encode to a list, and we use this fact to check which
        # serializer ran:
        assert bins == {'normal': 1234, 'tuple': [1,2,3]}
        self.delete_keys.append(key)

    def test_builtin_with_class_serializer_and_instance_serializer(self):
        """
            Invoke put() for mixed data record with python serializer.
        """
        aerospike.set_serializer((lambda v: json.dumps(v)))
        aerospike.set_deserializer((lambda v: json.loads(v)))
        hostlist, user, password = TestBaseClass.get_hosts()
        method_config = {
            'hosts': hostlist,
            'serialization': ((lambda v: marshal.dumps(v)), (lambda v: marshal.loads(v)))}
        if user == None and password == None:
            client = aerospike.client(method_config).connect()
        else:
            client = aerospike.client(method_config).connect(user, password)
        key = ('test', 'demo', 15)
        try:
            client.remove(key)
        except:
            pass

        rec = {'normal': 1234, 'tuple': (1,2,3)}
        res = client.put(key, rec, serializer=aerospike.SERIALIZER_PYTHON)

        assert res == 0

        _, _, bins = TestPythonSerializer.client.get(key)

        # tuples JSON-encode to a list, and we use this fact to check which
        # serializer ran:
        assert bins == {'normal': 1234, 'tuple': (1,2,3)}
        client.remove(key)


    def test_with_class_serializer_and_instance_serializer(self):
        """
            Invoke put() for mixed data record with class and instance serializer.
        """
        aerospike.set_serializer((lambda v: json.dumps(v)))
        aerospike.set_deserializer((lambda v: json.loads(v)))
        hostlist, user, password = TestBaseClass.get_hosts()
        method_config = {
            'hosts': hostlist,
            'serialization': ((lambda v: marshal.dumps(v)), (lambda v: marshal.loads(v)))}
        if user == None and password == None:
            client = aerospike.client(method_config).connect()
        else:
            client = aerospike.client(method_config).connect(user, password)
        key = ('test', 'demo', 16)
        try:
            TestPythonSerializer.client.remove(key)
        except:
            pass

        rec = {'normal': 1234, 'tuple': (1,2,3)}
        res = client.put(key, rec, serializer=aerospike.SERIALIZER_USER)

        assert res == 0

        _, _, bins = client.get(key)

        # tuples JSON-encode to a list, and we use this fact to check which
        # serializer ran:
        assert bins == {'normal': 1234, 'tuple': (1,2,3)}
        client.remove(key)

    def test_with_class_serializer_and_instance_serializer_with_unset_serializer(self):
        """
            Invoke put() for mixed data record with python serializer.
        """
        aerospike.set_serializer((lambda v: json.dumps(v)))
        aerospike.set_deserializer((lambda v: json.loads(v)))
        hostlist, user, password = TestBaseClass.get_hosts()
        method_config = {
            'hosts': hostlist
        }
        if user == None and password == None:
            client = aerospike.client(method_config).connect()
        else:
            client = aerospike.client(method_config).connect(user, password)
        key = ('test', 'demo', 16)
        try:
            TestPythonSerializer.client.remove(key)
        except:
            pass

        rec = {'normal': 1234, 'tuple': (1,2,3)}

        aerospike.unset_serializers()
        try:
            res = client.put(key, rec, serializer=aerospike.SERIALIZER_USER)
        except ClientError as exception:
            assert exception.code == -1
            assert exception.msg == "No serializer callback registered"

    def test_with_unset_serializer_python_serializer(self):
        """
            Invoke put() for mixed data record with python serializer and
            calling unset_serializers
        """
        aerospike.set_serializer((lambda v: json.dumps(v)))
        aerospike.set_deserializer((lambda v: json.loads(v)))
        hostlist, user, password = TestBaseClass.get_hosts()
        method_config = {
            'hosts': hostlist
        }
        if user == None and password == None:
            client = aerospike.client(method_config).connect()
        else:
            client = aerospike.client(method_config).connect(user, password)
        key = ('test', 'demo', 16)
        try:
            client.remove(key)
        except:
            pass

        rec = {'normal': 1234, 'tuple': (1,2,3)}

        aerospike.unset_serializers()
        res = client.put(key, rec, serializer=aerospike.SERIALIZER_PYTHON)
        
        assert res == 0

        _, _, bins = client.get(key)

        # tuples JSON-encode to a list, and we use this fact to check which
        # serializer ran:
        assert bins == {'normal': 1234, 'tuple': (1,2,3)}
        client.remove(key)



# -*- coding: utf-8 -*-

import pytest
import sys
import cPickle as pickle
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)
        
def handler(level, func, path, line, msg):
    assert 1 == 1

def extrahandler(level, func, myfile, line):
    print "Level is: %d" % level

class TestLog(object):
    def test_set_log_level_correct(self):
        """
        Test log level with correct parameters
        """

        response = aerospike.set_log_level(aerospike.LOG_LEVEL_DEBUG)

        assert response == 0

    def test_set_log_handler_correct(self):
        """
        Test log handler with correct parameters
        """

        response = aerospike.set_log_level(aerospike.LOG_LEVEL_DEBUG)
        aerospike.set_log_handler(handler)

        hostlist, user, password = TestBaseClass.get_hosts()
        config = {
                "hosts": hostlist
                }
        if user == None and password == None:
            client = aerospike.client(config).connect()
        else:
            client = aerospike.client(config).connect(user, password)

        assert response == 0
        client.close()

    def test_set_log_level_None(self):
        """
        Test log level with log level as None
        """
        try:
            response = aerospike.set_log_level(None)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'Invalid log level'

    def test_set_log_level_incorrect(self):
        """
        Test log level with log level incorrect
        """
        response = aerospike.set_log_level(9)

        assert response == 0

    def test_set_log_handler_extra_parameter(self):
        """
        Test log handler with extra parameter
        """

        aerospike.set_log_level(aerospike.LOG_LEVEL_DEBUG)

        with pytest.raises(TypeError) as typeError:
            aerospike.set_log_handler(handler, extrahandler)

        assert "setLogHandler() takes at most 1 argument (2 given)" in typeError.value



# -*- coding: utf-8 -*-

import pytest
import sys
import cPickle as pickle
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)


class TestSelect(TestBaseClass):
    def setup_class(cls):
        """
        Setup class.
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            TestSelect.client = aerospike.client(config).connect()
        else:
            TestSelect.client = aerospike.client(config).connect(user, password)

    def teardown_class(cls):
        TestSelect.client.close()

    def setup_method(self, method):
        """
        Setup method.
        """
        key = ('test', 'demo', 1)

        rec = {
            'a': ["nanslkdl", 1, bytearray("asd;as[d'as;d", "utf-8")],
            'b': {"key": "asd';q;'1';"},
            'c': 1234,
            'd': '!@#@#$QSDAsd;as',
            'n': None
        }

        TestSelect.client.put(key, rec)

    def teardown_method(self, method):
        """
        Teardoen method.
        """

        key = ("test", "demo", 1)

        TestSelect.client.remove(key)

    def test_select_with_key_and_empty_list_of_bins_to_select(self):

        key = ("test", "demo", 1)

        key, meta, bins = TestSelect.client.select(key, [])

        assert bins == {}

        assert meta != None

        assert key != None

    def test_select_with_key_and_bins(self):

        key = ("test", "demo", 1)

        bins_to_select = ['a']

        key, meta, bins = TestSelect.client.select(key, bins_to_select)

        assert bins == {
            'a': ["nanslkdl", 1, bytearray("asd;as[d'as;d", "utf-8")]
        }

        assert meta != None

        assert key != None

    def test_select_without_any_parameter(self):

        with pytest.raises(TypeError) as typeError:
            TestSelect.client.select()

        assert "Required argument 'key' (pos 1) not found" in typeError.value

    def test_select_with_none_key(self):

        bins_to_select = ['a']

        try:
            key, meta, bins = TestSelect.client.select(None, bins_to_select)

        except ParamError as exception:
            assert exception.code == -2

    def test_select_with_none_policy(self):

        key = ("test", "demo", 1)

        bins_to_select = ['b']

        key, meta, bins = TestSelect.client.select(key, bins_to_select, None)

        assert bins == {'b': {"key": "asd';q;'1';"}, }

        assert meta != None

        assert key != None

    def test_select_with_none_bins_to_select(self):

        key = ("test", "demo", 1)

        bins_to_select = None

        try:
            key, meta, bins = TestSelect.client.select( key, bins_to_select )

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'not a list or tuple'

    def test_select_with_non_existent_key(self):

        key = ("test", "demo", 'non-existent')

        bins_to_select = ['a', 'b']

        try:
            key, meta, bins = TestSelect.client.select( key, bins_to_select )

            """
            We are making the api backward compatible. In case of RecordNotFound an
            exception will not be raised. Instead Ok response is returned withe the
            meta as None. This might change with further releases.
            """
        except RecordNotFound as exception:
            assert True == False
            assert exception.code == 2
            assert exception.msg == 'AEROSPIKE_ERR_RECORD_NOT_FOUND'

        assert key != None
        assert meta == None
        assert bins == None

    def test_select_with_key_and_single_bin_to_select_not_a_list(self):

        key = ("test", "demo", 1)

        bin_to_select = 'a'  # Not a list

        try:
            key, meta, bins = TestSelect.client.select( key, bin_to_select )

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'not a list or tuple'

    def test_select_with_key_and_multiple_bins_to_select(self):

        key = ("test", "demo", 1)

        bins_to_select = ['c', 'd']

        key, meta, bins = TestSelect.client.select(key, bins_to_select)

        assert bins == {'c': 1234, 'd': '!@#@#$QSDAsd;as'}

        assert meta != None

    def test_select_with_key_and_multiple_bins_to_select_policy_key_send(self):

        key = ("test", "demo", 1)

        bins_to_select = ['c', 'd']
        policy = {'timeout': 1000, 'key': aerospike.POLICY_KEY_SEND}
        key, meta, bins = TestSelect.client.select(key, bins_to_select, policy)

        assert bins == {'c': 1234, 'd': '!@#@#$QSDAsd;as'}
        assert key == ('test', 'demo', 1, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )
        assert meta != None

    def test_select_with_key_and_multiple_bins_to_select_policy_key_digest(self
                                                                          ):

        key = ('test', 'demo', None, bytearray("asd;as[d'as;djk;uyfl",
                                               "utf-8"))
        rec = {
            'a': ["nanslkdl", 1, bytearray("asd;as[d'as;d", "utf-8")],
            'b': {"key": "asd';q;'1';"},
            'c': 1234,
            'd': '!@#@#$QSDAsd;as'
        }

        TestSelect.client.put(key, rec)

        bins_to_select = ['c', 'd']
        policy = {'timeout': 1000, 'key': aerospike.POLICY_KEY_DIGEST}
        key, meta, bins = TestSelect.client.select(key, bins_to_select, policy)

        assert bins == {'c': 1234, 'd': '!@#@#$QSDAsd;as'}
        assert key == ('test', 'demo', None,
                       bytearray(b"asd;as[d\'as;djk;uyfl"))
        assert meta != None

        key = ('test', 'demo', None, bytearray("asd;as[d'as;djk;uyfl",
                                               "utf-8"))
        TestSelect.client.remove(key)

    def test_select_with_key_and_combination_of_existent_and_non_existent_bins_to_select(
        self
    ):

        key = ("test", "demo", 1)

        bins_to_select = ['c', 'd', 'n']

        key, meta, bins = TestSelect.client.select(key, bins_to_select)

        assert bins == {'c': 1234, 'd': '!@#@#$QSDAsd;as', 'n': None}

        assert meta != None

    def test_select_with_key_and_non_existent_bin_in_middle(self):

        key = ("test", "demo", 1)

        bins_to_select = ['c', 'e', 'd']

        key, meta, bins = TestSelect.client.select(key, bins_to_select)

        assert bins == {'c': 1234, 'd': '!@#@#$QSDAsd;as'}

        assert meta != None

    def test_select_with_key_and_non_existent_bins_to_select(self):

        key = ("test", "demo", 1)

        bins_to_select = ['e', 'f']

        key, meta, bins = TestSelect.client.select(key, bins_to_select)

        assert bins == {}

    def test_select_with_unicode_value(self):

        key = ('test', 'demo', 'aa')

        rec = {
            'a': ["nanslkdl", 1, bytearray("asd;as[d'as;d", "utf-8")],
            'b': {"key": "asd';q;'1';"},
            'c': 1234,
            'd': '!@#@#$QSDAsd;as'
        }

        assert 0 == TestSelect.client.put(key, rec)

        bins_to_select = ['a']

        key, meta, bins = TestSelect.client.select(key, bins_to_select)

        assert bins == {
            'a': ["nanslkdl", 1, bytearray("asd;as[d'as;d", "utf-8")]
        }

        assert meta != None

        assert key != None

        key = ('test', 'demo', 'aa')
        TestSelect.client.remove(key)

    def test_select_with_key_and_bins_without_connection(self):

        key = ("test", "demo", 1)

        bins_to_select = ['a']

        try:
            key, meta, bins = TestSelect.client.select( key, bins_to_select)

        except ClusterError as exception:
            assert exception.code == 11L
            assert exception.msg == 'No connection to aerospike cluster'



# -*- coding: utf-8 -*-
import pytest
import time
import sys
import cPickle as pickle
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestGetRegistered(object):
    def setup_class(cls):
        """
        Setup class.
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            TestGetRegistered.client = aerospike.client(config).connect()
        else:
            TestGetRegistered.client = aerospike.client(config).connect(
                user, password)
        policy = {'timeout': 5000}
        TestGetRegistered.client.udf_put(u"bin_lua.lua", 0, policy)

    def teardown_class(cls):
        """
        Teardoen class.
        """
        policy = {'timeout': 5000}
        TestGetRegistered.client.udf_remove("bin_lua.lua", policy)
        TestGetRegistered.client.close()

    def test_udf_get_with_no_parameters(self):
        """
        Invoke udf_get() without any mandatory parameters.
        """
        with pytest.raises(TypeError) as typeError:
            TestGetRegistered.client.udf_get()

        assert "Required argument 'module' (pos 1) not found" in typeError.value

    def test_udf_get_with_correct_paramters(self):
        """
        Invoke udf_get() with correct parameters
        """
        module = "bin_lua.lua"
        language = aerospike.UDF_TYPE_LUA
        policy = {'timeout': 5000}

        udf_contents = TestGetRegistered.client.udf_get(module, language,
                                                        policy)

        #Check for udf file contents
        fo = open("bin_lua.lua", "r")
        contents = fo.read()
        assert contents == udf_contents
        fo.close()

    def test_udf_get_with_unicode_string_module_name(self):
        """
        Invoke udf_get() with correct parameters
        """
        module = u"bin_lua.lua"
        language = aerospike.UDF_TYPE_LUA
        policy = {'timeout': 5000}

        udf_contents = TestGetRegistered.client.udf_get(module, language,
                                                        policy)

        #Check for udf file contents
        fo = open("bin_lua.lua", "r")
        contents = fo.read()
        assert contents == udf_contents
        fo.close()

    def test_udf_get_with_correct_policy(self):
        """
        Invoke udf_get() with correct policy
        """
        module = "bin_lua.lua"
        language = aerospike.UDF_TYPE_LUA
        policy = {'timeout': 5000}
        udf_contents = TestGetRegistered.client.udf_get(module, language,
                                                        policy)

        #Check for udf file contents
        fo = open("bin_lua.lua", "r")
        contents = fo.read()
        assert contents == udf_contents
        fo.close()

    def test_udf_get_with_incorrect_policy(self):
        """
        Invoke udf_get() with incorrect policy
        """
        module = "bin_lua.lua"
        language = aerospike.UDF_TYPE_LUA
        policy = {'timeout': 0.5}

        try:
            TestGetRegistered.client.udf_get(module, language, policy)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "timeout is invalid"

    def test_udf_get_with_nonexistent_module(self):
        """
        Invoke udf_get() with non-existent module
        """
        module = "bin_transform_random"
        language = aerospike.UDF_TYPE_LUA
        policy = {'timeout': 1000}

        try:
            TestGetRegistered.client.udf_get(module, language, policy)

        except UDFError as exception:
            assert exception.code == 100
            assert exception.msg == "error=not_found\n"

    def test_udf_get_with_random_language(self):
        """
        Invoke udf_get() with random language
        """
        module = "bin_lua.lua"
        language = 85
        policy = {'timeout': 1000}

        try:
            TestGetRegistered.client.udf_get(module, language, policy)

        except ClientError as exception:
            assert exception.code == -1
            assert exception.msg == "Invalid language"

    def test_udf_get_with_extra_parameter(self):
        """
        Invoke udf_get() with extra parameter.
        """
        module = "bin_lua.lua"
        language = aerospike.UDF_TYPE_LUA
        policy = {'timeout': 1000}

        #Check for status or empty udf contents
        with pytest.raises(TypeError) as typeError:
            TestGetRegistered.client.udf_get(module, language, policy, "")

        assert "udf_get() takes at most 3 arguments (4 given)" in typeError.value

    def test_udf_get_policy_is_string(self):
        """
        Invoke udf_get() with policy is string
        """
        module = "bin_lua.lua"
        language = aerospike.UDF_TYPE_LUA

        #with pytest.raises(Exception) as exception:
        try:
            TestGetRegistered.client.udf_get(module, language, "")

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "policy must be a dict"

    def test_udf_get_module_is_none(self):
        """
        Invoke udf_get() with module is none
        """
        language = aerospike.UDF_TYPE_LUA

        try:
            TestGetRegistered.client.udf_get(None, language)

        except ClientError as exception:
            assert exception.code == -1
            assert exception.msg == "Module name should be a string or unicode string."
    
    def test_udf_get_with_unicode_module(self):
        """
        Invoke udf_get() with module name is unicode string
        """
        module = u"bin_lua.lua"
        language = aerospike.UDF_TYPE_LUA
        policy = {'timeout': 5000}

        udf_contents = TestGetRegistered.client.udf_get(module, language,
                                                        policy)

        #Check for udf file contents
        fo = open("bin_lua.lua", "r")
        contents = fo.read()
        assert contents == udf_contents
        fo.close()

    def test_udf_get_with_correct_paramters_without_connection(self):
        """
        Invoke udf_get() with correct parameters without connection
        """
        module = "bin_lua.lua"
        language = aerospike.UDF_TYPE_LUA
        policy = {'timeout': 5000}

        config = {'hosts': [('127.0.0.1', 3000)]}
        client1 = aerospike.client(config)

        try:
            udf_contents = client1.udf_get(module, language, policy)

        except ClusterError as exception:
            assert exception.code == 11L
            assert exception.msg == 'No connection to aerospike cluster'



# -*- coding: utf-8 -*-
import pytest
import time
import sys
import cPickle as pickle
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestScanInfo(object):
    def setup_method(self, method):
        """
        Setup method.
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            self.client = aerospike.client(config).connect()
        else:
            self.client = aerospike.client(config).connect(user, password)
        for i in xrange(5):
            key = ('test', 'demo', i)
            rec = {'name': 'name%s' % (str(i)), 'age': i}
            self.client.put(key, rec)
        policy = {}
        self.client.udf_put("bin_lua.lua", 0, policy)
        self.job_id = self.client.scan_apply("test", "demo", "bin_lua",
                                              "mytransform", ['age', 2])

    def teardown_method(self, method):
        """
        Teardoen method.
        """
        for i in xrange(5):
            key = ('test', 'demo', i)
            self.client.remove(key)
        self.client.close()

    def test_job_info_with_no_parameters(self):
        """
        Invoke job_info() without any mandatory parameters.
        """
        with pytest.raises(TypeError) as typeError:
            self.client.job_info()
        assert "Required argument 'job_id' (pos 1) not found" in typeError.value

    def test_job_info_with_correct_parameters(self):
        """
        Invoke job_info() with correct parameters
        """
        job_info = self.client.job_info(self.job_id, aerospike.JOB_SCAN)

        if job_info['status'] == aerospike.JOB_STATUS_COMPLETED or \
           job_info['status'] == aerospike.JOB_STATUS_UNDEF or \
           job_info['status'] or aerospike.JOB_STATUS_INPROGRESS:
            assert True == True
        else:
            assert True == False

    def test_job_info_with_correct_policy(self):
        """
        Invoke job_info() with correct policy
        """
        policy = {'timeout': 1000}
        job_info = self.client.job_info(self.job_id, aerospike.JOB_SCAN, policy)

        if job_info['status'] == aerospike.JOB_STATUS_COMPLETED or \
           job_info['status'] == aerospike.JOB_STATUS_UNDEF or \
           job_info['status'] or aerospike.JOB_STATUS_INPROGRESS:
            assert True == True
        else:
            assert True == False

    def test_job_info_with_incorrect_policy(self):
        """
        Invoke job_info() with incorrect policy
        """
        policy = {
            'timeout': 0.5
        }
        try:
            job_id = self.client.job_info(self.job_id, aerospike.JOB_SCAN, policy)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "timeout is invalid"

    def test_job_info_with_scanid_negative(self):
        """
        Invoke job_info() with scan id negative
        """
        with pytest.raises(TypeError) as typeError:
            job_info = self.client.job_info(-2)
        assert "Required argument 'module' (pos 2) not found" in typeError.value

    def test_job_info_with_scanid_incorrect(self):
        """
        Invoke job_info() with scan id incorrect
        """
        with pytest.raises(TypeError) as typeError:
            job_info = self.client.job_info(1)
        assert "Required argument 'module' (pos 2) not found" in typeError.value

    def test_job_info_with_scanid_string(self):
        """
        Invoke job_info() with scan id incorrect
        """
        with pytest.raises(TypeError) as typeError:
            job_info = self.client.job_info("string")
        assert "an integer is required" in typeError.value

    def test_job_info_with_correct_parameters_without_connection(self):
        """
        Invoke job_info() with correct parameters without connection
        """

        config = {'hosts': [('127.0.0.1', 3000)]}
        client1 = aerospike.client(config)

        try:
            job_info = client1.job_info(self.job_id, aerospike.JOB_SCAN)

        except ClusterError as exception:
            assert exception.code == 11L
            assert exception.msg == 'No connection to aerospike cluster'



# -*- coding: utf-8 -*-

import pytest
import sys
import time
import marshal
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    import aerospike
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)


class TestPut(TestBaseClass):
    def setup_class(cls):
        """
            Setup class
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {"hosts": hostlist}
        config_strict_types = {"hosts": hostlist, "strict_types": False}
        if user == None and password == None:
            TestPut.client = aerospike.client(config).connect()
            TestPut.client_strict_types = aerospike.client(config_strict_types).connect()
        else:
            TestPut.client = aerospike.client(config).connect(user, password)
            TestPut.client_strict_types = aerospike.client(config_strict_types).connect(user, password)
        TestPut.skip_old_server = True
        versioninfo = TestPut.client.info('version')
        for keys in versioninfo:
            for value in versioninfo[keys]:
                if value != None:
                    versionlist = value[value.find("build") + 6:value.find("\n")].split(".")
                    if int(versionlist[0]) >= 3 and int(versionlist[1]) >= 6:
                        TestPut.skip_old_server = False

    def teardown_class(cls):
        TestPut.client.close()
        TestPut.client_strict_types.close()

    def setup_method(self, method):
        """
            Setup method
        """
        self.delete_keys = []

    def teardown_method(self, method):
        """
            Teardown method
        """
        for key in self.delete_keys:
            TestPut.client.remove(key)


    def test_put_with_string_record(self):
        """
            Invoke put() for a record with string data.
        """
        key = ('test', 'demo', 1)

        bins = {"name": "John"}

        assert 0 == TestPut.client.put(key, bins)

        (key, meta, bins) = TestPut.client.get(key)

        assert {"name": "John"} == bins
        self.delete_keys.append(key)

    def test_put_with_multiple_bins(self):
        """
            Invoke put() with multiple bins and multiple types of data.
            Covers list, map, bytearray, integer.
        """
        key = ('test', 'demo', 1)

        bins = {
            'i': ["nanslkdl", 1, bytearray("asd;as[d'as;d", "utf-8")],
            's': {"key": "asd';q;'1';"},
            'b': 1234,
            'l': '!@#@#$QSDAsd;as'
        }

        assert 0 == TestPut.client.put(key, bins)
        (key, meta, bins) = TestPut.client.get(key)

        assert {
            'i': ["nanslkdl", 1, bytearray("asd;as[d'as;d", "utf-8")],
            's': {"key": "asd';q;'1';"},
            'b': 1234,
            'l': '!@#@#$QSDAsd;as'
        } == bins
        self.delete_keys.append(key)

    def test_put_with_no_parameters(self):
        """
            Invoke put() without any parameters.
        """
        with pytest.raises(TypeError) as typeError:
            res = TestPut.client.put()

        assert "Required argument 'key' (pos 1) not found" in typeError.value

    def test_put_without_record(self):
        """
            Invoke put() without any record data.
        """
        key = ('test', 'demo', 1)

        with pytest.raises(TypeError) as typeError:
            res = TestPut.client.put(key)

        assert "Required argument 'bins' (pos 2) not found" in typeError.value

    def test_put_with_none_key(self):
        """
            Invoke put() with None as key.
        """
        bins = {"name": "John"}

        try:
            res = TestPut.client.put(None, bins)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'key is invalid'

    def test_put_with_none_namespace_in_key(self):
        """
            Invoke put() with None namespace in key.
        """
        key = (None, "demo", 1)

        bins = {"name": "Steve"}

        try:
            TestPut.client.put(key, bins)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "namespace must be a string"

    def test_put_and_get_with_none_set_in_key(self):
        """
            Invoke put() with None set in key.
        """
        key = ("test", None, 1)

        bins = {"name": "John"}

        assert 0 == TestPut.client.put(key, bins)

        _, _, bins = TestPut.client.get(key)

        assert {"name": "John"} == bins

        self.delete_keys.append(key)

    def test_put_with_none_primary_key_in_key(self):
        """
            Invoke put() with None primary key in key.
        """
        key = ("test", "demo", None)

        bins = {"name": "John"}

        try:
            TestPut.client.put(key, bins)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "either key or digest is required"

    def test_put_with_bytearray_primary_key(self):
        """
            Invoke put() with bytearray primary key in key.
        """
        key = ("test", "demo", bytearray("asd;as[d'as;d", "utf-8"))

        bins = {"name": "John"}

        TestPut.client.put(key, bins)

        (key, meta, bins) = TestPut.client.get(key)

        assert bins == {"name": "John"}

        self.delete_keys.append(key)

    def test_put_with_string_type_record(self):
        """
            Invoke put() with string typed record.
        """
        key = ('test', 'demo', 15)

        kvs = "Name : John"

        try:
            res = TestPut.client.put( key, kvs )

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Record should be passed as bin-value pair"

    def test_put_with_wrong_ns_and_set(self):
        """
            Invoke put() with wrong ns and set.
        """
        key = ('demo', 'test', 1)

        bins = {'a': ['!@#!#$%#', bytearray('ASD@#$AR#$@#ERQ#', 'utf-8')]}

        try:
            res = TestPut.client.put( key, bins )

        except NamespaceNotFound as exception:
            assert exception.code == 20
            assert exception.msg == 'AEROSPIKE_ERR_NAMESPACE_NOT_FOUND'

    def test_put_with_nonexistent_namespace(self):
        """
            Invoke put() with non-existent namespace.
        """
        key = ('test1', 'demo', 1)

        bins = { 'i': 'asdadasd' }
        try:
            res = TestPut.client.put( key, bins )

        except NamespaceNotFound as exception:
            assert exception.code == 20
            assert exception.msg == 'AEROSPIKE_ERR_NAMESPACE_NOT_FOUND'

    def test_put_with_nonexistent_set(self):
        """
            Invoke put() with non-existent set.
        """
        key = ('test', 'unknown_set', 1)

        bins = {'a': {'k': [bytearray("askluy3oijs", "utf-8")]}}

        res = TestPut.client.put(key, bins)

        assert res == 0

        (key, meta, bins) = TestPut.client.get(key)

        assert bins == {'a': {'k': [bytearray(b'askluy3oijs')]}}

        self.delete_keys.append(key)

    def test_put_boolean_record(self):
        """
            Invoke put() for boolean data record.
        """
        key = ('test', 'demo', 1)

        bins = {"is_present": False}

        res = TestPut.client.put(key, bins)

        assert res == 0

        (key, meta, bins) = TestPut.client.get(key)

        assert bins == {"is_present": False}
        self.delete_keys.append(key)

    """ 
    def test_put_unicode_string(self):
            #Invoke put() for unicode record.
        key = ('test', 'demo', 1)

        bins = { "unicode_string": u"\ud83d\ude04" }

        res = TestPut.client.put( key, bins )

        assert res == 0

        (key , meta, bins) = TestPut.client.get(key)
        assert bins['unicode_string'] == u"\ud83d\ude04"
        self.delete_keys.append( key )
        #self.client.remove(key)
    
    def test_put_unicode_key(self):

        #    Invoke put() for unicode key.
        key = ('test', 'demo', u"\ud83d\ude04")
        rec = {
                "unicode_string": u"\ud83d\ude04"
        }

        res = TestPut.client.put( key, rec )

        assert res == 0

        (key , meta, bins) = TestPut.client.get(key)

        assert bins == rec

        key = ('test', 'demo', u"\ud83d\ude04")
        
        self.delete_keys.append( key )
    """

    def test_put_unicode_string_in_map(self):
        #Invoke put() for unicode record.
        key = ('test', 'demo', 1)

        rec = {'a': {u'aa': u'11'}, 'b': {u'bb': u'22'}}

        res = TestPut.client.put(key, rec)

        assert res == 0

        (key, meta, bins) = TestPut.client.get(key)
        assert bins == rec
        self.delete_keys.append(key)

    def test_put_unicode_string_in_list(self):
        #Invoke put() for unicode record.
        key = ('test', 'demo', 1)

        rec = {'a': [u'aa', u'bb', 1, u'bb', u'aa']}

        res = TestPut.client.put(key, rec)

        assert res == 0

        (key, meta, bins) = TestPut.client.get(key)

        assert bins == rec

        self.delete_keys.append(key)

    def test_put_unicode_string_in_key(self):
        #Invoke put() for unicode record.
        key = ('test', 'demo', u"bb")

        rec = {'a': [u'aa', 2, u'aa', 4, u'cc', 3, 2, 1]}
        res = TestPut.client.put(key, rec)

        assert res == 0

        (key, meta, bins) = TestPut.client.get(key)

        assert bins == rec

        self.delete_keys.append(key)

    def test_put_with_float_data(self):

        #Invoke put() for float data record.
        key = ('test', 'demo', 1)

        rec = {"pi": 3.141}

        res = TestPut.client.put(key, rec)

        assert res == 0
        _, _, bins = TestPut.client.get(key)

        assert bins == {'pi': 3.141}

        self.delete_keys.append(key)

    def test_put_with_float_data_within_list(self):

        #Invoke put() for float data record within list.
        key = ('test', 'demo', 1)

        rec = {"double_list": [3.141, 4.123, 6.285]}

        res = TestPut.client.put(key, rec)

        assert res == 0
        _, _, bins = TestPut.client.get(key)

        assert bins == {'double_list': [3.141, 4.123, 6.285]}

        self.delete_keys.append(key)

    def test_put_with_float_data_within_map(self):

        #Invoke put() for float data record within map.
        key = ('test', 'demo', 1)

        rec = {"double_map": {"1": 3.141,"2": 4.123,"3": 6.285}}

        res = TestPut.client.put(key, rec)

        assert res == 0
        _, _, bins = TestPut.client.get(key)

        assert bins == {'double_map': {"1": 3.141, "2": 4.123, "3": 6.285}}

        self.delete_keys.append(key)

    def test_put_with_string_meta_and_string_policies(self):
        """
            Invoke put() for metadata and policies.
        """
        key = ('test', 'demo', 1)

        rec = {'i': 12}

        try:
            res = TestPut.client.put( key, rec, "meta", "policies")

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "policy must be a dict"

    def test_put_with_string_record_generation(self):
        """
            Invoke put() for a record with string data, metadata and ttl
        """
        key = ('test', 'demo', 1)

        rec = {"name": "John"}
        meta = {'gen': 3, 'ttl': 25000}
        policy = {'timeout': 1000}
        assert 0 == TestPut.client.put(key, rec, meta, policy)

        (key, meta, bins) = TestPut.client.get(key)
        assert {"name": "John"} == bins
        assert meta['gen'] != None
        self.delete_keys.append(key)

    def test_put_with_generation_string(self):
        """
            Invoke put() for a record with generation as string
        """
        key = ('test', 'demo', 1)

        rec = {"name": "John"}
        meta = {'gen': "wrong", 'ttl': 25000}
        policy = {'timeout': 1000}

        try:
            TestPut.client.put( key, rec, meta, policy )

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Generation should be an int or long"

        #self.delete_keys.append( key )

    def test_put_with_ttl_string(self):
        """
            Invoke put() for a record with ttl as string
        """
        key = ('test', 'demo', 1)

        rec = {
                "name" : "John"
                }
        meta = {
            'gen': 3,
            'ttl': "25000"
        }
        policy = {
            'timeout': 1000
        }
        try:
            TestPut.client.put( key, rec, meta, policy )

        except ParamError as exception:
            assert exception.code == -2
            #assert exception.msg == "TTL should be an int or long"

        #self.delete_keys.append( key )

    def test_put_with_generation_bool(self):
        """
            Invoke put() for a record with generation as boolean.
        """
        key = ('test', 'demo', 1)

        rec = {"name": "John"}
        meta = {'gen': True, 'ttl': 25000}
        policy = {'timeout': 1000}
        assert 0 == TestPut.client.put(key, rec, meta, policy)

        (key, meta, bins) = TestPut.client.get(key)
        assert {"name": "John"} == bins
        assert meta['gen'] != None
        self.delete_keys.append(key)

    def test_put_with_ttl_boolean(self):
        """
            Invoke put() for a record with ttl as boolean.
        """
        key = ('test', 'demo', 1)

        rec = {"name": "John"}
        meta = {'gen': 3, 'ttl': True}
        policy = {'timeout': 1000}
        assert 0 == TestPut.client.put(key, rec, meta, policy)

        (key, meta, bins) = TestPut.client.get(key)
        assert {"name": "John"} == bins
        assert meta['gen'] != None
        self.delete_keys.append(key)

    def test_put_with_policy_timeout_string(self):
        """
            Invoke put() for a record with policy timeout as string
        """
        key = ('test', 'demo', 1)

        rec = {
                "name" : "John"
                }
        meta = {
            'gen': 3,
            'ttl' :25000
        }
        policy = {
            'timeout': "1000"
        }
        try:
            TestPut.client.put( key, rec, meta, policy )

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'timeout is invalid'

    def test_put_with_policy_gen_EQ_positive(self):
        """
            Invoke put() for a record with generation as EQ positive
        """
        key = ('test', 'demo', 1)

        rec = {"name": "John"}
        meta = {'gen': 2, 'ttl': 25000}
        policy = {'timeout': 1000}
        assert 0 == TestPut.client.put(key, rec, meta, policy)

        (key, meta, bins) = TestPut.client.get(key)

        assert {"name": "John"} == bins
        gen = meta['gen']
        rec = {"name": "Smith"}
        policy = {
            'timeout': 1000,
            'gen': aerospike.POLICY_GEN_EQ,
            'commit_level': aerospike.POLICY_COMMIT_LEVEL_ALL
        }
        meta = {'gen': gen}

        assert 0 == TestPut.client.put(key, rec, meta, policy)

        (key, meta, bins) = TestPut.client.get(key)
        assert {"name": "Smith"} == bins

        self.delete_keys.append(key)

    def test_put_with_policy_gen_EQ_less(self):
        """
            Invoke put() for a record with generation as EQ less
        """
        key = ('test', 'demo', 1)

        rec = {"name": "John"}
        meta = {'gen': 2, 'ttl': 25000}
        policy = {'timeout': 1000}
        assert 0 == TestPut.client.put(key, rec, meta, policy)

        (key, meta, bins) = TestPut.client.get(key)

        assert {"name": "John"} == bins
        gen = meta['gen']
        rec = {"name": "Smith"}
        policy = {'timeout': 1000, 'gen': aerospike.POLICY_GEN_EQ}
        meta = {'gen': 10}

        try:
            TestPut.client.put( key, rec, meta, policy )

        except RecordGenerationError as exception:
            assert exception.code == 3
            assert exception.msg == 'AEROSPIKE_ERR_RECORD_GENERATION'
        
        ( key, meta, bins) = TestPut.client.get(key)
        assert {"name": "John"} == bins

        self.delete_keys.append(key)

    def test_put_with_policy_exists_create_negative(self):
        """
            Invoke put() for a record with all policies.
        """
        key = ('test', 'demo', 1)

        rec = {"name": "John"}
        meta = {'gen': 2, 'ttl': 25000}
        policy = {
            'timeout': 1000,
            'gen': aerospike.POLICY_GEN_IGNORE,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'key': aerospike.POLICY_KEY_SEND,
        }
        assert 0 == TestPut.client.put(key, rec, meta, policy)

        (key, meta, bins) = TestPut.client.get(key)

        assert {"name": "John"} == bins

        rec = {"name": "Smith"}
        policy = {'timeout': 1000, 'exists': aerospike.POLICY_EXISTS_CREATE}
        meta = {'gen': 2}

        try:
            TestPut.client.put( key, rec, meta, policy )

        except RecordExistsError as exception:
            assert exception.code == 5
            assert exception.msg == 'AEROSPIKE_ERR_RECORD_EXISTS'
            assert exception.bin == {'name': 'Smith'}

        (key, meta, bins) = TestPut.client.get(key)
        assert {"name": "John"} == bins

        self.delete_keys.append(key)

    def test_put_with_policy_exists_create_positive(self):
        """
            Invoke put() for a record with all policies.
        """
        key = ('test', 'demo', 1)

        rec = {"name": "Smith"}
        meta = {'gen': 2, 'ttl': 25000}
        policy = {
            'timeout': 1000,
            'exists': aerospike.POLICY_EXISTS_CREATE,
            'gen': aerospike.POLICY_GEN_IGNORE,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'key': aerospike.POLICY_KEY_SEND,
            'commit_level': aerospike.POLICY_COMMIT_LEVEL_MASTER
        }
        assert 0 == TestPut.client.put(key, rec, meta, policy)

        (key, meta, bins) = TestPut.client.get(key)
        assert {"name": "Smith"} == bins

        self.delete_keys.append(key)

    def test_put_with_policy_exists_replace_negative(self):
        """
            Invoke put() for a record with replace policy negative.
        """
        key = ('test', 'demo', 1)

        rec = {"name": "John"}
        meta = {'gen': 2, 'ttl': 25000}
        policy = {
            'timeout': 1000,
            'exists': aerospike.POLICY_EXISTS_REPLACE,
            'gen': aerospike.POLICY_GEN_IGNORE,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'key': aerospike.POLICY_KEY_SEND
        }
        try:
            assert 0 == TestPut.client.put( key, rec, meta, policy )

        except RecordNotFound as exception:
            assert exception.code == 2
            assert exception.msg == 'AEROSPIKE_ERR_RECORD_NOT_FOUND'

        #self.delete_keys.append( key )

    def test_put_with_policy_exists_create_or_replace_positive(self):
        """
            Invoke put() for a record with create or replace policy positive.
        """
        key = ('test', 'demo', 1)

        rec = {"name": "Smith"}
        meta = {'gen': 2, 'ttl': 25000}
        policy = {
            'timeout': 1000,
            'exists': aerospike.POLICY_EXISTS_CREATE_OR_REPLACE,
            'gen': aerospike.POLICY_GEN_IGNORE,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'key': aerospike.POLICY_KEY_SEND
        }
        assert 0 == TestPut.client.put(key, rec, meta, policy)

        (key, meta, bins) = TestPut.client.get(key)
        assert {"name": "Smith"} == bins

        self.delete_keys.append(key)

    def test_put_with_policy_exists_ignore(self):
        """
            Invoke put() for a record with ignore.
        """
        key = ('test', 'demo', 1)

        rec = {"name": "Smith"}
        meta = {'gen': 2, 'ttl': 25000}
        policy = {
            'timeout': 1000,
            'exists': aerospike.POLICY_EXISTS_IGNORE,
            'gen': aerospike.POLICY_GEN_IGNORE,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'key': aerospike.POLICY_KEY_SEND
        }
        assert 0 == TestPut.client.put(key, rec, meta, policy)

        (key, meta, bins) = TestPut.client.get(key)
        assert {"name": "Smith"} == bins

        self.delete_keys.append(key)

    def test_put_with_policy_replace_positive(self):
        """
            Invoke put() for a record with replace positive.
        """
        key = ('test', 'demo', 1)

        rec = {"name": "John"}
        meta = {'gen': 2, 'ttl': 25000}
        policy = {
            'timeout': 1000,
            'gen': aerospike.POLICY_GEN_IGNORE,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'key': aerospike.POLICY_KEY_SEND,
        }
        assert 0 == TestPut.client.put(key, rec, meta, policy)

        (key, meta, bins) = TestPut.client.get(key)

        assert {"name": "John"} == bins

        rec = {"name": "Smith"}
        meta = {'gen': 2, 'ttl': 25000}
        policy = {'timeout': 1000, 'exists': aerospike.POLICY_EXISTS_REPLACE}
        assert 0 == TestPut.client.put(key, rec, meta, policy)

        (key, meta, bins) = TestPut.client.get(key)

        assert {"name": "Smith"} == bins
        self.delete_keys.append(key)

    def test_put_with_policy_exists_update_positive(self):
        """
            Invoke put() for a record with all policies.
        """
        key = ('test', 'demo', 1)

        rec = {"name": "John"}
        meta = {'gen': 2, 'ttl': 25000}
        policy = {
            'timeout': 1000,
            'gen': aerospike.POLICY_GEN_IGNORE,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'key': aerospike.POLICY_KEY_SEND,
        }
        assert 0 == TestPut.client.put(key, rec, meta, policy)

        (key, meta, bins) = TestPut.client.get(key)

        assert {"name": "John"} == bins

        rec = {"name": "Smith"}
        meta = {'gen': 2, 'ttl': 25000}
        policy = {'timeout': 1000, 'exists': aerospike.POLICY_EXISTS_UPDATE}
        assert 0 == TestPut.client.put(key, rec, meta, policy)

        (key, meta, bins) = TestPut.client.get(key)

        assert {"name": "Smith"} == bins
        self.delete_keys.append(key)

    def test_put_with_policy_exists_update_negative(self):
        """
            Invoke put() for a record with update policy negative.
        """
        key = ('test', 'demo', 1)

        rec = {"name": "John"}
        meta = {'gen': 2, 'ttl': 25000}
        policy = {
            'timeout': 1000,
            'exists': aerospike.POLICY_EXISTS_UPDATE,
            'gen': aerospike.POLICY_GEN_IGNORE,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'key': aerospike.POLICY_KEY_SEND
        }
        try:
            assert 0 == TestPut.client.put( key, rec, meta, policy )

        except RecordNotFound as exception:
            assert exception.code == 2
            assert exception.msg == 'AEROSPIKE_ERR_RECORD_NOT_FOUND'

        #self.delete_keys.append( key )
    def test_put_with_policy_gen_GT_lesser(self):
        """
            Invoke put() for a record with generation as GT lesser
        """
        key = ('test', 'demo', 1)

        rec = {"name": "John"}
        meta = {'gen': 2, 'ttl': 25000}
        policy = {'timeout': 1000}
        assert 0 == TestPut.client.put(key, rec, meta, policy)

        (key, meta, bins) = TestPut.client.get(key)

        assert {"name": "John"} == bins
        gen = meta['gen']
        rec = {"name": "Smith"}
        policy = {'timeout': 1000, 'gen': aerospike.POLICY_GEN_GT}
        meta = {'gen': gen}

        try:
            TestPut.client.put( key, rec, meta, policy )

        except RecordGenerationError as exception:
            assert exception.code == 3
            assert exception.msg == 'AEROSPIKE_ERR_RECORD_GENERATION'

        (key, meta, bins) = TestPut.client.get(key)
        assert {"name": "John"} == bins

        self.delete_keys.append(key)

    def test_put_with_policy_gen_GT_positive(self):
        """
            Invoke put() for a record with generation as GT positive
        """
        key = ('test', 'demo', 1)

        rec = {"name": "John"}
        meta = {'gen': 2, 'ttl': 25000}
        policy = {'timeout': 1000}
        assert 0 == TestPut.client.put(key, rec, meta, policy)

        (key, meta, bins) = TestPut.client.get(key)

        assert {"name": "John"} == bins
        gen = meta['gen']
        assert gen == 1
        rec = {"name": "Smith"}
        policy = {'timeout': 1000, 'gen': aerospike.POLICY_GEN_GT}
        meta = {'gen': gen + 5}

        TestPut.client.put(key, rec, meta, policy)

        (key, meta, bins) = TestPut.client.get(key)
        assert {"name": "Smith"} == bins

        self.delete_keys.append(key)

    def test_put_with_policy_gen_ignore(self):
        """
            Invoke put() for a record with generation as gen_ignore
        """
        key = ('test', 'demo', 1)

        rec = {"name": "John"}
        meta = {'gen': 2, 'ttl': 25000}
        policy = {'timeout': 1000}
        assert 0 == TestPut.client.put(key, rec, meta, policy)

        (key, meta, bins) = TestPut.client.get(key)

        assert {"name": "John"} == bins

        gen = meta['gen']
        rec = {"name": "Smith"}
        policy = {'timeout': 1000, 'gen': aerospike.POLICY_GEN_IGNORE}
        meta = {'gen': gen}

        assert 0 == TestPut.client.put(key, rec, meta, policy)

        (key, meta, bins) = TestPut.client.get(key)
        assert {"name": "Smith"} == bins

        self.delete_keys.append(key)

    def test_put_with_set_unicode_string(self):
        """
            Invoke put() with set is unicode string.
        """
        key = ('test', u'demo', 1)

        rec = {"name": "John"}

        assert 0 == TestPut.client.put(key, rec)

        (key, meta, bins) = TestPut.client.get(key)

        assert {"name": "John"} == bins
        self.delete_keys.append(key)

    def test_put_with_unicode_bin(self):
        """
            Invoke put() with unicode bin.
        """
        key = ('test', 'demo', 1)

        rec = {
            u'i': ["nanslkdl", 1, bytearray("asd;as[d'as;d", "utf-8")],
            's': {"key": "asd';q;'1';"},
            'b': 1234,
            'l': '!@#@#$QSDAsd;as'
        }

        assert 0 == TestPut.client.put(key, rec)
        (key, meta, bins) = TestPut.client.get(key)

        assert {
            'i': ["nanslkdl", 1, bytearray("asd;as[d'as;d", "utf-8")],
            's': {"key": "asd';q;'1';"},
            'b': 1234,
            'l': '!@#@#$QSDAsd;as'
        } == bins
        self.delete_keys.append(key)

    def test_put_set(self):
        """
            Invoke put() set.
        """
        key = ('test', 'demo', 1)

        rec = {"is_present": set([1, 2])}

        res = TestPut.client.put(key, rec)

        assert res == 0

        (key, meta, bins) = TestPut.client.get(key)

        assert bins == {"is_present": set([1, 2])}

        self.delete_keys.append(key)

    def test_put_frozenset(self):
        """
            Invoke put() frozenSet.
        """
        key = ('test', 'demo', 1)

        cities = frozenset(["Frankfurt", "Basel", "Freiburg"])

        rec = {'fSet': cities}

        res = TestPut.client.put(key, rec)

        assert res == 0

        (key, meta, bins) = TestPut.client.get(key)

        assert bins == {'fSet': frozenset(["Frankfurt", "Basel", "Freiburg"])}

        self.delete_keys.append(key)

    def test_put_tuple(self):
        """
            Invoke put() tuple.
        """
        key = ('test', 'demo', 1)

        rec = {'seq': tuple('abc')}

        res = TestPut.client.put(key, rec)

        assert res == 0

        (key, meta, bins) = TestPut.client.get(key)

        assert bins == {'seq': ('a', 'b', 'c')}

        self.delete_keys.append(key)

    def test_put_none_data(self):
        """
            Invoke put() None.
        """
        key = ('test', 'demo', 1)

        rec_none = {"is_present": None}

        res = TestPut.client.put(key, rec_none)

        assert res == 0

        (key, meta, bins) = TestPut.client.get(key)

        assert bins == {"is_present": None}
        self.delete_keys.append(key)

    def test_put_map_containing_tuple(self):
        """
            Invoke put() maap containing tuple.
        """
        key = ('test', 'demo', 1)

        rec = {'seq': {'bb': tuple('abc')}}

        res = TestPut.client.put(key, rec)

        assert res == 0

        (key, meta, bins) = TestPut.client.get(key)

        assert bins == {'seq': {u'bb': ('a', 'b', 'c')}}

        self.delete_keys.append(key)

    def test_put_serializer_default(self):
        """
            Invoke put() with mixed data record with no class or instance
            serializer or deserializer. Python option should get called by default
        """

        key = ('test', 'demo', 1)

        rec = {
            'map': {"key": "asd';q;'1';",
                    "pi": 3.14},
            'normal': 1234,
            'special': '!@#@#$QSDAsd;as',
            'list': ["nanslkdl", 1, bytearray("asd;as[d'as;d", "utf-8")],
            'bytes': bytearray("asd;as[d'as;d", "utf-8"),
            'nestedlist': ["nanslkdl", 1, bytearray("asd;as[d'as;d", "utf-8"),
                           [1, bytearray("asd;as[d'as;d", "utf-8")]],
            'nestedmap': {
                "key": "asd';q;'1';",
                "pi": 3.14,
                "nest": {"pi1": 3.12,
                         "t": 1}
            },
        }

        res = TestPut.client.put(key, rec, {}, {})

        assert res == 0

        _, _, bins = TestPut.client.get(key)

        assert bins == {
            'map': {"key": "asd';q;'1';",
                    "pi": 3.14},
            'normal': 1234,
            'special': '!@#@#$QSDAsd;as',
            'list': ["nanslkdl", 1, bytearray("asd;as[d'as;d", "utf-8")],
            'bytes': bytearray("asd;as[d'as;d", "utf-8"),
            'nestedlist': ["nanslkdl", 1, bytearray("asd;as[d'as;d", "utf-8"),
                           [1, bytearray("asd;as[d'as;d", "utf-8")]],
            'nestedmap':
            {"key": "asd';q;'1';",
             "pi": 3.14,
             "nest": {"pi1": 3.12,
                      "t": 1}},
        }
        self.delete_keys.append(key)

    def test_put_user_serializer_no_deserializer(self):
        """
            Invoke put() for float data record with user serializer is
            registered, but deserializer is not registered.
        """

        key = ('test', 'demo', 1)

        rec = {"pi": 3.14}

        def serialize_function(val):
            return marshal.dumps(val)

        response = aerospike.set_serializer(serialize_function)

        res = TestPut.client.put(key, rec, {}, {}, aerospike.SERIALIZER_USER)

        assert res == 0

        _, _, bins = TestPut.client.get(key)

        if TestPut.skip_old_server == False:
            assert bins == {'pi': 3.14}
        else:
            assert bins == {'pi': bytearray(b'g\x1f\x85\xebQ\xb8\x1e\t@')}

        self.delete_keys.append(key)

    def test_put_record_with_bin_name_exceeding_max_limit(self):
        """
            Invoke put() with bin name exceeding the max limit of bin name.
        """
        key = ('test', 'demo', 'put_rec')
        put_record = {
            'containers_free': [],
            'containers_used': [
                {'cluster_id': 'bob',
                 'container_id': 1,
                 'port': 4000}
            ],
            'list_of_map': [{'test': 'bar'}],
            'map_of_list': {'fizz': ['b', 'u', 'z', 'z']},
            'ports_free': [],
            'ports_unused': [4100, 4200, 4300],
            'provider_id': u'i-f01fc206'
        }

        try:
            TestPut.client.put( key, put_record)

        except BinNameError as exception:
            assert exception.code == 21L
            assert exception.msg == "A bin name should not exceed 14 characters limit"

    def test_put_with_string_record_without_connection(self):
        """
            Invoke put() for a record with string data without connection
        """
        config = {"hosts": [("127.0.0.1", 3000)]}
        client1 = aerospike.client(config)

        key = ('test', 'demo', 1)

        bins = {"name": "John"}

        try:
            client1.put( key, bins )

        except ClusterError as exception:
            assert exception.code == 11L
            assert exception.msg == 'No connection to aerospike cluster'

    def test_put_with_integer_greater_than_maxisze(self):
        """
            Invoke put() for a record with integer greater than max size
        """
        key = ('test', 'demo', 1)

        bins = {"no": 111111111111111111111111111111111111111111111}

        try:
            assert 0 == TestPut.client.put(key, bins)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'integer value exceeds sys.maxsize'

    def test_put_with_integer_no_exception_raised_CLIENT598(self):
        """
            Invoke put() for a record with integer equal to -1. No exception
            raised. Test for CLIENT-598
        """
        key = ('test', 'demo', 1)

        bins = {"no": -1L}

        TestPut.client.put(key, bins)

        (key, meta, bins) = TestPut.client.get(key)

        assert bins == {"no": -1}
        self.delete_keys.append(key)

    def test_put_with_key_as_an_integer_greater_than_maxisze(self):
        """
            Invoke put() for a record with integer greater than max size
        """
        key = ('test', 'demo', 1111111111111111111111111111111111)

        bins = {"no": 11}

        try:
            assert 0 == TestPut.client.put(key, bins)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'integer value for KEY exceeds sys.maxsize'

    def test_put_record_set_to_aerospike_null(self):
        """
            Invoke put() for a record with bin set to aerospike_null
        """
        key = ('test', 'demo', 1)

        bins = {"name": "John", "no": 3}

        assert 0 == TestPut.client.put(key, bins)

        (key, meta, bins) = TestPut.client.get(key)

        assert {"name": "John", "no": 3} == bins

        bins = {"no": aerospike.null}

        assert 0 == TestPut.client.put(key, bins)

        (key, meta, bins) = TestPut.client.get(key)

        assert {"name": "John"} == bins

        self.delete_keys.append(key)

    def test_put_strict_types_bin_length(self):
        """
            Invoke put() for a record with strict type set to false and bin
            length more than 14 characters
        """
        key = ('test', 'demo', 1)

        maxlength = ""
        for i in xrange(20):
            maxlength = maxlength + "a"

        bins = {"name": "John", maxlength: 3}

        assert 0 == TestPut.client_strict_types.put(key, bins)

        (key, meta, bins) = TestPut.client_strict_types.get(key)

        assert {"name": "John"} == bins
        self.delete_keys.append(key)

    def test_put_strict_types_empty_record(self):
        """
            Invoke put() for a record with strict type set to false and record
            is empty
        """
        key = ('test', 'demo', 1)

        bins = {}

        assert 0 == TestPut.client_strict_types.put(key, bins)

        (key, meta, bins) = TestPut.client_strict_types.get(key)

        assert None == bins



# -*- coding: utf-8 -*-

import pytest
import sys
import time
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestRevokePrivilege(TestBaseClass):

    pytestmark = pytest.mark.skipif(
        TestBaseClass().get_hosts()[1] == None,
        reason="No user specified, may be not secured cluster.")

    def setup_method(self, method):

        """
        Setup method
        """
        hostlist, user, password = TestBaseClass().get_hosts()
        config = {
                "hosts": hostlist
                }
        self.client = aerospike.client(config).connect( user, password )
        try:
            self.client.admin_drop_role("usr-sys-admin-test")
        except:
            pass
        self.client.admin_create_role("usr-sys-admin-test", [{"code":
            aerospike.PRIV_USER_ADMIN}, {"code": aerospike.PRIV_SYS_ADMIN}])
        self.delete_users = []

    def teardown_method(self, method):

        """
        Teardown method
        """
        policy = {}

        self.client.admin_drop_role("usr-sys-admin-test")
        self.client.close()

    def test_admin_revoke_privileges_no_parameters(self):
        """
            Revoke privileges with no parameters
        """
        with pytest.raises(TypeError) as typeError:
            self.client.admin_revoke_privileges()

        assert "Required argument 'role' (pos 1) not found" in typeError.value

    def test_admin_revoke_privileges_positive(self):
        """
            revoke privileges positive
        """
        status = self.client.admin_grant_privileges("usr-sys-admin-test", [{"code":
aerospike.PRIV_READ}])

        assert status == 0
        time.sleep(1)
        roles = self.client.admin_query_role("usr-sys-admin-test")
        assert roles == [{'code': 0, 'ns': '', 'set': ''},
{'code': 1, 'ns': '', 'set': ''}, {'code': 10, 'ns': '', 'set': ''}]

        status = self.client.admin_revoke_privileges("usr-sys-admin-test", [{"code":
            aerospike.PRIV_READ}])

        assert status == 0
        time.sleep(1)
        roles = self.client.admin_query_role("usr-sys-admin-test")
        assert roles == [{'code': 0, 'ns': '', 'set': ''},
{'code': 1, 'ns': '', 'set': ''}]

    def test_admin_revoke_privileges_positive_with_policy(self):
        """
            Revoke privileges positive with policy
        """
        status = self.client.admin_grant_privileges("usr-sys-admin-test", [{"code":
aerospike.PRIV_READ}], {'timeout': 1000})

        assert status == 0
        time.sleep(1)
        roles = self.client.admin_query_role("usr-sys-admin-test")
        assert roles == [{'code': 0, 'ns': '', 'set': ''},
{'code': 1, 'ns': '', 'set': ''}, {'code': 10, 'ns': '', 'set': ''}]

        status = self.client.admin_revoke_privileges("usr-sys-admin-test", [{"code":
            aerospike.PRIV_READ}], {'timeout': 1000})
        time.sleep(1)
        assert status == 0
        roles = self.client.admin_query_role("usr-sys-admin-test")
        assert roles == [{'code': 0, 'ns': '', 'set': ''},
{'code': 1, 'ns': '', 'set': ''}]

    def test_admin_revoke_privileges_positive_with_ns_set(self):
        """
            Revoke privileges positive with ns and set
        """
        status = self.client.admin_grant_privileges("usr-sys-admin-test", [{"code":
aerospike.PRIV_READ, "ns":"test", "set":"demo"}])

        assert status == 0
        time.sleep(1)
        roles = self.client.admin_query_role("usr-sys-admin-test")
        assert roles == [{'code': 0, 'ns': '', 'set': ''},
{'code': 1, 'ns': '', 'set': ''}, {'code': 10, 'ns': 'test', 'set': 'demo'}]

        status = self.client.admin_revoke_privileges("usr-sys-admin-test", [{"code":
aerospike.PRIV_READ, "ns":"test", "set":"demo"}])
        time.sleep(1)
        assert status == 0
        roles = self.client.admin_query_role("usr-sys-admin-test")
        assert roles == [{'code': 0, 'ns': '', 'set': ''},
{'code': 1, 'ns': '', 'set': ''}]

    def test_revoke_privileges_incorrect_role_type(self):
        """
            role name not string
        """
        try:
            self.client.admin_revoke_privileges(1, [{"code": aerospike.PRIV_USER_ADMIN}])

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Role name should be a string"

    def test_revoke_privileges_unknown_privilege_type(self):
        """
            privilege type unknown
        """
        try:
            self.client.admin_revoke_privileges("usr-sys-admin-test", [{"code": 64}])
        except InvalidPrivilege as exception:
            assert exception.code == 72

    def test_revoke_privileges_incorrect_privilege_type(self):
        """
            privilege type incorrect
        """
        try:
            self.client.admin_revoke_privileges("usr-sys-admin-test", None)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Privileges should be a list"

    def test_revoke_privileges_empty_list_privileges(self):
        """
            privilege type is an empty list
        """
        try:
            self.client.admin_revoke_privileges("usr-sys-admin-test", [])

        except InvalidPrivilege as exception:
            assert exception.code == 72
            assert exception.msg == 'AEROSPIKE_INVALID_PRIVILEGE'



# -*- coding: utf-8 -*-

import pytest
import sys
import cPickle as pickle
import time
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
from aerospike import predicates as p
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)



class TestQuery(TestBaseClass):
    def setup_class(cls):
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            client = aerospike.client(config).connect()
        else:
            client = aerospike.client(config).connect(user, password)

        client.index_integer_create('test', 'demo', 'test_age',
                                              'age_index')
        client.index_string_create('test', 'demo', 'addr',
                                             'addr_index')
        client.index_integer_create('test', 'demo', 'age1',
                                              'age_index1')
        client.index_list_create('test', 'demo', 'numeric_list',
                                           aerospike.INDEX_NUMERIC,
                                           'numeric_list_index')
        client.index_list_create('test', 'demo', 'string_list',
                                           aerospike.INDEX_STRING,
                                           'string_list_index')
        client.index_map_keys_create('test', 'demo', 'numeric_map',
                                               aerospike.INDEX_NUMERIC,
                                               'numeric_map_index')
        client.index_map_keys_create('test', 'demo', 'string_map',
                                               aerospike.INDEX_STRING,
                                               'string_map_index')
        client.index_map_values_create('test', 'demo', 'numeric_map',
                                                 aerospike.INDEX_NUMERIC,
                                                 'numeric_map_values_index')
        client.index_map_values_create('test', 'demo', 'string_map',
                                                 aerospike.INDEX_STRING,
                                                 'string_map_values_index')
        client.index_integer_create('test', None, 'test_age_none', 
                                                'age_index_none')
        client.index_integer_create('test', 'demo', bytearray("sal\0kj", "utf-8"),
                                             'sal_index')

    def teardown_class(cls):
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            client = aerospike.client(config).connect()
        else:
            client = aerospike.client(config).connect(user, password)
        policy = {}
        client.index_remove('test', 'age_index', policy)
        client.index_remove('test', 'age_index1', policy)
        client.index_remove('test', 'addr_index', policy)
        client.index_remove('test', 'numeric_list_index', policy)
        client.index_remove('test', 'string_list_index', policy)
        client.index_remove('test', 'numeric_map_index', policy)
        client.index_remove('test', 'string_map_index', policy)
        client.index_remove('test', 'numeric_map_values_index', policy)
        client.index_remove('test', 'string_map_values_index', policy)
        client.index_remove('test', 'age_index_none', policy);
        client.index_remove('test', 'sal_index')
        client.close()

    def setup_method(self, method):
        """
        Setup method.
        """
        config = {'hosts': TestBaseClass.hostlist}
        if TestBaseClass.user == None and TestBaseClass.password == None:
            self.client = aerospike.client(config).connect()
        else:
            self.client = aerospike.client(config).connect(
                TestBaseClass.user, TestBaseClass.password)

        for i in xrange(5):
            key = ('test', 'demo', i)
            rec = {
                'name': 'name%s' % (str(i)),
                'addr': 'name%s' % (str(i)),
                'numeric_list': [i, i + 1, i + 2],
                'string_list': ["str" + str(i), "str" + str(i + 1),
                                "str" + str(i + 2)],
                'numeric_map': {"a": i,
                                "b": i + 1,
                                "c": i + 2},
                'string_map': {
                    "a": "a" + str(i),
                    "b": "b" + str(i + 1),
                    "c": "c" + str(i + 2)
                },
                'test_age': i,
                'no': i
            }
            self.client.put(key, rec)
        for i in xrange(5, 10):
            key = ('test', 'demo', i)
            rec = {
                u'name': 'name%s' % (str(i)),
                u'addr': u'name%s' % (str(i)),
                u'test_age': i,
                u'no': i
            }
            self.client.put(key, rec)

        key = ('test', 'demo', 122)
        list = [{"op": aerospike.OPERATOR_WRITE,
                "bin": bytearray("sal\0kj", "utf-8"),
                "val": 80000}];
        self.client.operate(key, list)

    def teardown_method(self, method):
        """
        Teardown method.
        """
        for i in xrange(10):
            key = ('test', 'demo', i)
            self.client.remove(key)

        key = ('test', 'demo', 122)
        self.client.remove(key)
        self.client.close()

    def test_query_with_no_parameters(self):
        """
            Invoke query() without any mandatory parameters.
        """
        try:
            query = self.client.query()
            query.select()
            query.where()

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'query() expects atleast 1 parameter'

    def test_query_with_correct_parameters(self):
        """
            Invoke query() with correct arguments
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.equals('test_age', 1))

        records = []

        def callback((key, metadata, record)):
            records.append(key)

        query.foreach(callback)
        assert records
        assert len(records) == 1

    def test_query_with_incorrect_ns_set(self):
        """
            Invoke query() with incorrect ns and set
        """
        try:
            query = self.client.query('test1', 'demo1')
            query.select('name', 'test_age')
            query.where(p.equals('test_age', 1))

            def callback((key, metadata, record)):
                assert metadata['gen'] != None

            query.foreach(callback)

        except InvalidRequest as exception:
            assert exception.code == 4L
            assert exception.msg == 'AEROSPIKE_ERR_REQUEST_INVALID'

    def test_query_with_ns_not_string(self):
        """
            Invoke query() with incorrect ns and set
        """
        try:
            query = self.client.query(1, 'demo')
            query.select('name', 'test_age')
            query.where(p.equals('test_age', 1))
            def callback((key,metadata,record)):
                assert metadata['gen'] != None

            query.foreach(callback)

        except ParamError as exception:
            assert exception.code == -2L
            assert exception.msg == 'Namespace should be a string'

    def test_query_with_set_int(self):
        """
            Invoke query() with incorrect ns and set
        """
        try:
            query = self.client.query('test', 1)
            query.select('name', 'test_age')
            query.where(p.equals('test_age', 1))
            def callback((key,metadata,record)):
                assert metadata['gen'] != None

            query.foreach(callback)

        except ParamError as exception:
            assert exception.code == -2L
            assert exception.msg == 'Set should be string, unicode or None'

    def test_query_with_incorrect_bin_name(self):
        """
            Invoke query() with incorrect bin name
        """
        query = self.client.query('test', 'demo')
        query.select('name1', 'age1')
        query.where(p.equals('age1', 1))
        records = []

        def callback((key, metadata, record)):
            records.append(record)

        query.foreach(callback)
        assert len(records) == 0

    def test_query_without_callback_parameter(self):
        """
            Invoke query() with without callback
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.equals('test_age', 1))

        def callback((key, metadata, record)):
            assert metadata['gen'] != None

        with pytest.raises(TypeError) as typeError:
            query.foreach()

        assert "Required argument 'callback' (pos 1) not found" in typeError.value

    def test_query_with_nonindexed_bin(self):
        """
            Invoke query() with non-indexed bin
        """
        #with pytest.raises(Exception) as exception:
        try:
            query = self.client.query('test', 'demo')
            query.select('name', 'no')
            query.where(p.equals('no', 1))

            def callback((key, metadata, record)):
                assert metadata['gen'] != None

            query.foreach(callback)

        except IndexNotFound as exception:
            assert exception.code == 201
            assert exception.msg == 'AEROSPIKE_ERR_INDEX_NOT_FOUND'
            assert exception.name == None

    def test_query_with_where_incorrect(self):
        """
            Invoke query() with where is incorrect
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.equals('test_age', 165))
        records = []

        def callback((key, metadata, record)):
            records.append(record)

        query.foreach(callback)
        assert len(records) == 0

    def test_query_with_where_none_value(self):
        """
            Invoke query() with where is null value
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        try:
            query.where(p.equals('test_age', None))

        except ParamError as exception:
            assert exception.code == -2L
            assert exception.msg == 'predicate is invalid.'

    def test_query_with_policy(self):
        """
            Invoke query() with policy
        """
        policy = {'timeout': 1000}
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.equals('test_age', 1))
        records = []

        def callback((key, metadata, record)):
            records.append(record)

        query.foreach(callback, policy)
        assert len(records) == 1

    def test_query_with_extra_argument(self):
        """
            Invoke query() with extra argument
        """
        policy = {'timeout': 1000}
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.equals('test_age', 1))

        def callback((key, metadata, record)):
            assert metadata['gen'] != None

        with pytest.raises(TypeError) as typeError:
            query.foreach(callback, policy, "")

        assert "foreach() takes at most 2 arguments (3 given)" in typeError.value

    def test_query_with_incorrect_policy_value(self):
        """
            Invoke query() with incorrect policy
        """
        policy = {'timeout': ""}
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.equals('test_age', 1))

        def callback((key, metadata, record)):
            assert metadata['gen'] != None

        try:
            query.foreach(callback, policy)

        except ParamError as exception:
            assert exception.code == -2L
            assert exception.msg == 'timeout is invalid'

    def test_query_with_put_in_callback(self):
        """
            Invoke query() with put in callback
        """
        policy = {'timeout': 1000}
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.equals('test_age', 1))
        records = []

        def callback((key, metadata, record)):
            records.append(record)
            key = ('test', 'demo', 'put_in_callback')
            rec = {'name': 'name%s' % (str(8)), 'test_age': 8, }
            self.client.put(key, rec)

        query.foreach(callback, policy)

        key = ('test', 'demo', 'put_in_callback')
        key1, meta, bins = self.client.get(key)

        key = ('test', 'demo', 'put_in_callback')
        self.client.remove(key)
        assert bins == {'test_age': 8, 'name': 'name8'}

    def test_query_with_correct_parameters_between(self):
        """
            Invoke query() with correct arguments and using predicate between
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.between('test_age', 1, 4))

        records = []

        def callback((key, metadata, record)):
            records.append(record)

        query.foreach(callback)
        assert len(records) == 4

    def test_query_with_where_is_null(self):

        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        try:
            query.where("")

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "predicate is invalid."

    def test_query_with_callback_contains_error(self):
        """
            Invoke query() with callback function contains an error
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.equals('test_age', 1))

        records = []

        def callback((key, metadata, record)):
            val += 1
            records.append(key)

        try:
            result = query.foreach(callback)

        except ClientError as exception:
            assert exception.code == -1L
            assert exception.msg == "Callback function contains an error"

    def test_query_with_callback_returning_false(self):
        """
            Invoke query() with callback function returns false
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.between('test_age', 1, 5))

        records = []

        def callback((key, metadata, record)):
            if len(records) == 2:
                return False
            records.append(key)

        result = query.foreach(callback)
        assert len(records) == 2

    def test_query_with_results_method(self):
        """
            Invoke query() with correct arguments
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.equals('test_age', 1))

        records = []
        records = query.results()
        assert len(records) == 1

    def test_query_with_unicode_binnames_in_select_and_where(self):
        """
            Invoke query() with unicode bin names in select
        """
        query = self.client.query('test', 'demo')
        query.select(u'name', u'test_age', 'addr')
        query.where(p.equals(u'test_age', 7))

        records = query.results()
        assert len(records) == 1
        assert records[0][2] == {
            'test_age': 7,
            'name': u'name7',
            'addr': u'name7'
        }

        query = self.client.query('test', 'demo')
        query.select(u'name', 'addr')
        query.where(p.equals(u'addr', u'name9'))

        records = query.results()
        assert records[0][2] == {'name': 'name9', 'addr': u'name9'}

    def test_query_with_select_bin_integer(self):
        """
            Invoke query() with select bin is of integer type.
        """
        query = self.client.query('test', 'demo')

        try:
            query.select(22, 'test_age')

        except ParamError as exception:
            assert exception.code == -2L
            assert exception.msg == 'Bin name should be of type string'

    def test_query_with_correct_parameters_contains(self):
        """
            Invoke query() with correct arguments and using predicate contains
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.contains('numeric_list', aerospike.INDEX_TYPE_LIST, 1))

        records = []

        def callback((key, metadata, record)):
            records.append(record)

        query.foreach(callback)
        assert len(records) == 2

    def test_query_with_correct_parameters_containsstring(self):
        """
            Invoke query() with correct arguments and using predicate contains
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.contains('string_list', aerospike.INDEX_TYPE_LIST,
                               "str3"))

        records = []

        def callback((key, metadata, record)):
            records.append(record)

        query.foreach(callback)
        assert len(records) == 3

    def test_query_with_correct_parameters_rangecontains(self):
        """
            Invoke query() with correct arguments and using predicate contains
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.range('numeric_list', aerospike.INDEX_TYPE_LIST, 1, 3))

        records = []

        def callback((key, metadata, record)):
            records.append(record)

        query.foreach(callback)
        assert len(records) == 2

    def test_query_with_correct_parameters_containsstring_mapkeys(self):
        """
            Invoke query() with correct arguments and using predicate contains
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.contains('string_map', aerospike.INDEX_TYPE_MAPKEYS, "a"))

        records = []

        def callback((key, metadata, record)):
            records.append(record)

        query.foreach(callback)
        assert len(records) == 5

    def test_query_with_correct_parameters_containsstring_mapvalues(self):
        """
            Invoke query() with correct arguments and using predicate contains
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.contains('string_map', aerospike.INDEX_TYPE_MAPVALUES,
                               "a1"))

        records = []

        def callback((key, metadata, record)):
            records.append(record)

    def test_query_with_multiple_foreach_on_same_query_object(self):
        """
            Invoke query() with multple foreach() call on same query object
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.equals('test_age', 1))

        records = []

        def callback((key, metadata, record)):
            records.append(key)

        query.foreach(callback)
        assert len(records) == 1

    def test_query_with_correct_parameters_containsnumeric_mapvalues(self):
        """
            Invoke query() with correct arguments and using predicate contains
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.contains('numeric_map', aerospike.INDEX_TYPE_MAPVALUES,
                               1))

        records = []

        def callback((key, metadata, record)):
            records.append(record)

        query.foreach(callback)
        assert len(records) == 2

    def test_query_with_correct_parameters_rangecontains(self):
        """
            Invoke query() with correct arguments and using predicate contains
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where(p.range('numeric_map', aerospike.INDEX_TYPE_MAPVALUES, 1,
                            3))

        records = []

        def callback((key, metadata, record)):
            records.append(record)

        query.foreach(callback)
        assert len(records) == 8

    def test_query_with_correct_parameters_rangecontains_notuple(self):
        """
            Invoke query() with correct arguments and using predicate contains
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where('numeric_map', "range", aerospike.INDEX_TYPE_MAPVALUES,
                    aerospike.INDEX_NUMERIC, 1, 3)

        records = []

        def callback((key, metadata, record)):
            records.append(record)

        query.foreach(callback)
        assert len(records) == 8

    def test_query_with_correct_parameters_containsstring_mapvalues_notuple(
        self
    ):
        """
            Invoke query() with correct arguments and using predicate contains
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where('string_map', 'contains', aerospike.INDEX_TYPE_MAPVALUES,
                    aerospike.INDEX_STRING, "a1")

        records = []

        def callback((key, metadata, record)):
            records.append(record)

        query.foreach(callback)
        assert len(records) == 1

    def test_query_with_correct_parameters_containsstring_notuple(self):
        """
            Invoke query() with correct arguments and using predicate contains
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where('string_list', "contains", aerospike.INDEX_TYPE_LIST,
                    aerospike.INDEX_STRING, "str3")

        records = []

        def callback((key, metadata, record)):
            records.append(record)

        query.foreach(callback)
        assert len(records) == 3

    def test_query_with_correct_parameters_between_notuple(self):
        """
            Invoke query() with correct arguments and using predicate between
        """
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where('test_age', 'between', 1, 4)

        records = []

        def callback((key, metadata, record)):
            records.append(record)

        query.foreach(callback)
        assert len(records) == 4

    def test_query_with_policy_notuple(self):
        """
            Invoke query() with policy
        """
        policy = {'timeout': 1000}
        query = self.client.query('test', 'demo')
        query.select('name', 'test_age')
        query.where('test_age', 'equals', 1)
        records = []

        def callback((key, metadata, record)):
            records.append(record)

        query.foreach(callback, policy)
        assert len(records) == 1
        records = []
        query.foreach(callback)
        assert len(records) == 1

    def test_query_with_multiple_results_call_on_same_query_object(self):
        """
            Invoke query() with multple results() call on same query object
        """
        query = self.client.query('test', 'demo')
        query.select(u'name', u'test_age', 'addr')
        query.where(p.equals(u'test_age', 7))

        records = query.results()
        assert len(records) == 1
        assert records[0][2] == {
            'test_age': 7,
            'name': u'name7',
            'addr': u'name7'
        }

        records = []
        records = query.results()
        assert len(records) == 1
        assert records[0][2] == {
            'test_age': 7,
            'name': u'name7',
            'addr': u'name7'
        }

    def test_query_with_correct_parameters_without_connection(self):
        """
            Invoke query() with correct arguments without connection
        """
        config = {'hosts': [('127.0.0.1', 3000)]}
        client1 = aerospike.client(config)

        try:
            query = client1.query('test', 'demo')
            query.select('name', 'test_age')
            query.where(p.equals('test_age', 1))

            records = []

            def callback((key, metadata, record)):
                records.append(key)

            query.foreach(callback)

        except ClusterError as exception:
            assert exception.code == 11L
            assert exception.msg == 'No connection to aerospike cluster'

    def test_query_with_policy_on_none_set_index(self):
        """
            Invoke query() with policy on none set index
        """
        policy = {
            'timeout': 1000
        }
        query = self.client.query('test', None)
        query.select('name', 'test_age_none')
        query.where(p.equals('test_age_none', 1))
        records = []
        def callback((key,metadata,record)):
            records.append(record)

        query.foreach(callback, policy)
        assert len(records) == 0

    def test_query_with_only_ns(self):
        """
            Invoke query() with policy on none set index
        """
        policy = {
            'timeout': 1000
        }
        query = self.client.query('test')
        query.select('name', 'test_age_none')
        query.where(p.equals('test_age_none', 1))
        records = []
        def callback((key,metadata,record)):
            records.append(record)

        query.foreach(callback, policy)
        assert len(records) == 0

    def test_query_with_select_bytearray(self):
        """
            Invoke query() with correct arguments
        """
        query = self.client.query('test', 'demo')
        query.select(bytearray("sal\0kj", "utf-8"))
        query.where(p.equals(bytearray("sal\0kj", "utf-8"), 80000))

        records = []

        def callback((key, metadata, record)):
            records.append(key)

        query.foreach(callback)
        assert records
        assert len(records) == 1



import pytest
import ConfigParser


class TestBaseClass(object):
    hostlist = []
    user = None
    password = None
    has_ldt = None
    has_geo = None

    @staticmethod
    def get_hosts():
        config = ConfigParser.ConfigParser()
        config.read("config.conf")
        if config.has_option('enterprise-edition', 'hosts'):
            hosts_str = config.get('enterprise-edition','hosts')
            if hosts_str != 'None':
                TestBaseClass.hostlist = TestBaseClass.parse_hosts(hosts_str)
        if len(TestBaseClass.hostlist) > 0:
            if config.has_option('enterprise-edition', 'user'):
                TestBaseClass.user = config.get('enterprise-edition','user')
            if config.has_option('enterprise-edition', 'password'):
                TestBaseClass.password = config.get('enterprise-edition','password')
        else:
            TestBaseClass.hostlist = TestBaseClass.parse_hosts(
                    config.get('community-edition','hosts'))
        return TestBaseClass.hostlist, TestBaseClass.user, TestBaseClass.password

    @staticmethod
    def parse_hosts(host_str):
        hosts = []
        host_str = host_str.strip()
        hlist = host_str.split(';')
        for host in hlist:
            host = host.strip().split(':')
            if len(host) == 2:
                host[1] = int(host[1])
                hosts.append(tuple(host))
        return hosts

    @staticmethod
    def has_ldt_support():
        if TestBaseClass.has_ldt is not None:
            return TestBaseClass.has_ldt
        import aerospike
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            client = aerospike.client(config).connect()
        else:
            client = aerospike.client(config).connect(user, password)
        response = client.info('get-config:context=namespace;id=test', config['hosts'])
        namespace_config = response.values()[0][1]
        if namespace_config.find('ldt-enabled=true') == -1:
            TestBaseClass.has_ldt = False
        else:
            TestBaseClass.has_ldt = True
        client.close()
        return TestBaseClass.has_ldt

    @staticmethod
    def has_geo_support():
        if TestBaseClass.has_geo is not None:
            return TestBaseClass.has_geo
        import aerospike
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            client = aerospike.client(config).connect()
        else:
            client = aerospike.client(config).connect(user, password)
        TestBaseClass.has_geo = client.has_geo()
        client.close()
        return TestBaseClass.has_geo






# -*- coding: utf-8 -*-
import pytest
import time
import sys
import cPickle as pickle
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestIncrement(object):
    def setup_class(cls):
        """
        Setup method.
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            TestIncrement.client = aerospike.client(config).connect()
        else:
            TestIncrement.client = aerospike.client(config).connect(user,
                                                                    password)
        TestIncrement.skip_old_server = True
        versioninfo = TestIncrement.client.info('version')
        for keys in versioninfo:
            for value in versioninfo[keys]:
                if value != None:
                    versionlist = value[value.find("build") + 6:value.find("\n")].split(".")
                    if int(versionlist[0]) >= 3 and int(versionlist[1]) >= 6:
                        TestIncrement.skip_old_server = False

    def teardown_class(cls):
        TestIncrement.client.close()

    def setup_method(self, method):
        for i in xrange(5):
            key = ('test', 'demo', i)
            rec = {'name': 'name%s' % (str(i)), 'age': i}
            TestIncrement.client.put(key, rec)
        key = ('test', 'demo', 6)
        rec = {'age': 6.5}
        TestIncrement.client.put(key, rec)

    def teardown_method(self, method):
        """
        Teardoen method.
        """
        for i in xrange(5):
            key = ('test', 'demo', i)
            TestIncrement.client.remove(key)

    def test_increment_with_no_parameters(self):
        """
        Invoke increment() without any mandatory parameters.
        """
        with pytest.raises(TypeError) as typeError:
            TestIncrement.client.increment()
        assert "Required argument 'key' (pos 1) not found" in typeError.value

    def test_increment_with_correct_parameters(self):
        """
        Invoke increment() with correct parameters
        """
        key = ('test', 'demo', 1)
        TestIncrement.client.increment(key, "age", 5)

        (key, meta, bins) = TestIncrement.client.get(key)

        assert bins == {'age': 6, 'name': 'name1'}

    def test_increment_with_correct_parameters_float_value(self):
        """
        Invoke increment() with correct parameters and a float value
        """
        if TestIncrement.skip_old_server == True:
            pytest.skip("Server does not support increment on float type")
        key = ('test', 'demo', 6)
        TestIncrement.client.increment(key, "age", 6.4)

        (key, meta, bins) = TestIncrement.client.get(key)

        assert bins == {'age': 12.9}

    def test_increment_with_policy_key_send(self):
        """
        Invoke increment() with policy key send
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_SEND,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'commit_level': aerospike.POLICY_COMMIT_LEVEL_MASTER
        }
        TestIncrement.client.increment(key, "age", 5, {}, policy)

        (key, meta, bins) = TestIncrement.client.get(key)

        assert bins == {'age': 6, 'name': 'name1'}
        assert key == ('test', 'demo', None, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_increment_with_policy_key_digest(self):
        """
        Invoke increment() with policy key digest
        """
        key = ('test', 'demo', None, bytearray("asd;as[d'as;djk;uyfl",
                                               "utf-8"))
        rec = {'name': 'name%s' % (str(1)), 'age': 1, 'nolist': [1, 2, 3]}
        TestIncrement.client.put(key, rec)

        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_DIGEST,
            'retry': aerospike.POLICY_RETRY_NONE
        }
        TestIncrement.client.increment(key, "age", 5, {}, policy)

        (key, meta, bins) = TestIncrement.client.get(key)

        assert bins == {'age': 6, 'name': 'name1', 'nolist': [1, 2, 3]}
        assert key == ('test', 'demo', None,
                       bytearray(b"asd;as[d\'as;djk;uyfl"))
        TestIncrement.client.remove(key)

    def test_increment_with_correct_policy(self):
        """
        Invoke increment() with correct policy
        """
        key = ('test', 'demo', 1)
        policy = {'timeout': 1000, 'key': aerospike.POLICY_KEY_SEND}
        TestIncrement.client.increment(key, "age", 5, {}, policy)

        (key, meta, bins) = TestIncrement.client.get(key)

        assert bins == {'age': 6, 'name': 'name1'}

    def test_increment_with_policy_key_gen_EQ_ignore(self):
        """
        Invoke increment() with gen eq positive ignore
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_SEND,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'gen': aerospike.POLICY_GEN_IGNORE
        }

        meta = {'gen': 10, 'ttl': 1200}
        TestIncrement.client.increment(key, "age", 5, meta, policy)

        (key, meta, bins) = TestIncrement.client.get(key)

        assert bins == {'age': 6, 'name': 'name1'}
        assert key == ('test', 'demo', None, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_increment_with_policy_key_gen_EQ_positive(self):
        """
        Invoke increment() with gen eq positive
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_SEND,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'gen': aerospike.POLICY_GEN_EQ
        }
        (key, meta) = TestIncrement.client.exists(key)

        gen = meta['gen']
        meta = {'gen': gen, 'ttl': 1200}
        TestIncrement.client.increment(key, "age", 5, meta, policy)

        (key, meta, bins) = TestIncrement.client.get(key)

        assert bins == {'age': 6, 'name': 'name1'}
        assert key == ('test', 'demo', None, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_increment_with_policy_key_gen_EQ_not_equal(self):
        """
        Invoke increment() with policy key EQ not equal
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_SEND,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'gen': aerospike.POLICY_GEN_EQ
        }
        (key, meta) = TestIncrement.client.exists(key)
        gen = meta['gen']

        meta = {
            'gen': gen + 5,
            'ttl': 1200
        }
        try:
            TestIncrement.client.increment(key, "age", 5, meta, policy)

        except RecordGenerationError as exception:
            assert exception.code == 3
            assert exception.msg == "AEROSPIKE_ERR_RECORD_GENERATION"
            assert exception.bin == "age"

        (key , meta, bins) = TestIncrement.client.get(key)

        assert bins == {'age': 1, 'name': 'name1'}
        assert key == ('test', 'demo', None, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_increment_with_policy_key_gen_GT_lesser(self):
        """
        Invoke increment() with gen GT lesser
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_SEND,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'gen': aerospike.POLICY_GEN_GT
        }
        (key, meta) = TestIncrement.client.exists(key)

        gen = meta['gen']
        meta = {
            'gen': gen,
            'ttl': 1200
        }
        try:
            TestIncrement.client.increment(key, "age", 5, meta, policy)

        except RecordGenerationError as exception:
            assert exception.code == 3
            assert exception.msg == "AEROSPIKE_ERR_RECORD_GENERATION"

        (key, meta, bins) = TestIncrement.client.get(key)

        assert bins == {'age': 1, 'name': 'name1'}
        assert key == ('test', 'demo', None, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_increment_with_policy_key_gen_GT_positive(self):
        """
        Invoke increment() with gen GT positive
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_SEND,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'gen': aerospike.POLICY_GEN_GT
        }
        (key, meta) = TestIncrement.client.exists(key)

        gen = meta['gen']
        meta = {'gen': gen + 5, 'ttl': 1200}
        TestIncrement.client.increment(key, "age", 5, meta, policy)

        (key, meta, bins) = TestIncrement.client.get(key)

        assert bins == {'age': 6, 'name': 'name1'}
        assert key == ('test', 'demo', None, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_increment_with_incorrect_policy(self):
        """
        Invoke increment() with incorrect policy
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 0.5
        }
        try:
            TestIncrement.client.increment(key, "age", 5, {}, policy)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "timeout is invalid"

    def test_increment_with_nonexistent_key(self):
        """
        Invoke increment() with non-existent key
        """
        key = ('test', 'demo', 'non-existentkey')
        status = TestIncrement.client.increment(key, "age", 5)

        (key, meta, bins) = TestIncrement.client.get(key)

        assert bins == {'age': 5}

        TestIncrement.client.remove(key)

    def test_increment_with_nonexistent_bin(self):
        """
        Invoke increment() with non-existent bin
        """
        key = ('test', 'demo', 1)
        status = TestIncrement.client.increment(key, "age1", 5)

        (key, meta, bins) = TestIncrement.client.get(key)

        assert bins == {'age1': 5, 'name': u'name1', 'age': 1}

    def test_increment_with_extra_parameter(self):
        """
        Invoke increment() with extra parameter.
        """
        key = ('test', 'demo', 1)
        policy = {'timeout': 1000}
        with pytest.raises(TypeError) as typeError:
            TestIncrement.client.increment(key, "age", 2, {}, policy, "")

        assert "increment() takes at most 5 arguments (6 given)" in typeError.value

    def test_increment_policy_is_string(self):
        """
        Invoke increment() with policy is string
        """
        key = ('test', 'demo', 1)
        try:
            TestIncrement.client.increment(key, "age", 2, {}, "")

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "policy must be a dict"

    def test_increment_key_is_none(self):
        """
        Invoke increment() with key is none
        """
        try:
            TestIncrement.client.increment(None, "age", 2)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "key is invalid"

    def test_increment_bin_is_none(self):
        """
        Invoke increment() with bin is none
        """
        key = ('test', 'demo', 1)
        try:
            TestIncrement.client.increment(key, None, 2)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Bin name should be of type string"

    def test_increment_with_unicode_bin(self):
        """
        Invoke increment() with bin is unicode string
        """
        key = ('test', 'demo', 1)
        TestIncrement.client.increment(key, u"age", 10)

        (key, meta, bins) = TestIncrement.client.get(key)

        assert bins == {'age': 11, 'name': 'name1'}

    def test_increment_with_correct_parameters_without_connection(self):
        """
        Invoke increment() with correct parameters without connection
        """
        key = ('test', 'demo', 1)
        config = {'hosts': [('127.0.0.1', 3000)]}
        client1 = aerospike.client(config)

        try:
            client1.increment(key, "age", 5)

        except ClusterError as exception:
            assert exception.code == 11L
            assert exception.msg == 'No connection to aerospike cluster'

    def test_increment_with_integer_greaterthan_maxsize(self):
        """
        Invoke increment() with integer greater then(2^63 - 1)
        """
        key = ('test', 'demo', 1)
        bins = {"age": 10}
        TestIncrement.client.put(key, bins)
        try:
            TestIncrement.client.increment(key, 'age', 68786586756785785745)
        except Exception as exception:
            assert exception.code == -2
            assert exception.msg == 'integer value exceeds sys.maxsize'

    def test_increment_with_string_value(self):
        """
        Invoke increment() with string value
        """
        try:
            key = ('test', 'demo', 1)
            TestIncrement.client.increment(key, "age", "5")

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Unsupported operand type(s) for +: only 'int' allowed"




# -*- coding: utf-8 -*-

import pytest
import sys
from test_base_class import TestBaseClass
try:
    from collections import Counter
except ImportError:
    from counter26 import Counter

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestExistsMany(TestBaseClass):
    def setup_class(cls):
        """
        Setup method.
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            TestExistsMany.client = aerospike.client(config).connect()
        else:
            TestExistsMany.client = aerospike.client(config).connect(user,
                                                                     password)

    def teardown_class(cls):
        TestExistsMany.client.close()

    def setup_method(self, method):

        self.keys = []

        for i in xrange(5):
            key = ('test', 'demo', i)
            rec = {'name': 'name%s' % (str(i)), 'age': i}
            TestExistsMany.client.put(key, rec)
            self.keys.append(key)

    def teardown_method(self, method):
        """
        Teardown method.
        """
        for i in xrange(5):
            key = ('test', 'demo', i)
            TestExistsMany.client.remove(key)

    def test_exists_many_without_any_parameter(self):

        with pytest.raises(TypeError) as typeError:
            TestExistsMany.client.exists_many()

        assert "Required argument 'keys' (pos 1) not found" in typeError.value

    def test_exists_many_without_policy(self):

        records = TestExistsMany.client.exists_many(self.keys)

        assert type(records) == list
        assert len(records) == 5

    def test_exists_many_with_proper_parameters(self):

        records = TestExistsMany.client.exists_many(self.keys, {'timeout': 1200})

        assert type(records) == list
        assert len(records) == 5
        assert Counter([x[0][2] for x in records]) == Counter([0, 1, 2, 3,
            4])

    def test_exists_many_with_none_policy(self):

        records = TestExistsMany.client.exists_many(self.keys, None)

        assert type(records) == list
        assert len(records) == 5
        assert Counter([x[0][2] for x in records]) == Counter([0, 1, 2, 3,
            4])

    def test_exists_many_with_none_keys(self):

        try:
            TestExistsMany.client.exists_many( None, {} )

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Keys should be specified as a list or tuple."

    def test_exists_many_with_non_existent_keys(self):

        self.keys.append(('test', 'demo', 'some_key'))

        records = TestExistsMany.client.exists_many(self.keys)

        assert type(records) == list
        assert len(records) == 6
        assert Counter([x[0][2] for x in records]) == Counter([0, 1, 2, 3,
            4, 'some_key'])
        for x in records:
            if x[0][2] == 'some_key':
                assert x[1] == None

    def test_exists_many_with_all_non_existent_keys(self):

        keys = [('test', 'demo', 'key')]

        records = TestExistsMany.client.exists_many(keys)

        assert len(records) == 1
        for x in records:
            if x[0][2] == 'key':
                assert x[1] == None

    def test_exists_many_with_invalid_key(self):

        try:
            records = TestExistsMany.client.exists_many( "key" )

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Keys should be specified as a list or tuple."

    def test_exists_many_with_invalid_timeout(self):

        policies = { 'timeout' : 0.2 }
        try:
            records = TestExistsMany.client.exists_many(self.keys, policies)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "timeout is invalid"

    def test_exists_many_with_initkey_as_digest(self):

        keys = []
        key = ("test", "demo", None, bytearray("asd;as[d'as;djk;uyfl"))
        rec = {'name': 'name1', 'age': 1}
        TestExistsMany.client.put(key, rec)
        keys.append(key)

        key = ("test", "demo", None, bytearray("ase;as[d'as;djk;uyfl"))
        rec = {'name': 'name2', 'age': 2}
        TestExistsMany.client.put(key, rec)
        keys.append(key)

        records = TestExistsMany.client.exists_many(keys)

        for key in keys:
            TestExistsMany.client.remove(key)

        assert type(records) == list
        assert len(records) == 2
        i = 0
        for x in records:
            if i:
                assert x[0][3] == bytearray(b"ase;as[d'as;djk;uyfl")
            else:
                assert x[0][3] == bytearray(b"asd;as[d'as;djk;uyfl")
            i += 1

    def test_exists_many_with_non_existent_keys_in_middle(self):

        self.keys.append(('test', 'demo', 'some_key'))

        for i in xrange(15, 20):
            key = ('test', 'demo', i)
            rec = {'name': 'name%s' % (str(i)), 'age': i}
            TestExistsMany.client.put(key, rec)
            self.keys.append(key)

        records = TestExistsMany.client.exists_many(self.keys)

        for i in xrange(15, 20):
            key = ('test', 'demo', i)
            TestExistsMany.client.remove(key)

        assert type(records) == list
        assert len(records) == 11
        assert Counter([x[0][2] for x in records]) == Counter([0, 1, 2, 3,
            4, 'some_key', 15, 16, 17, 18, 19])
        for x in records:
            if x[0][2] == 'some_key':
                assert x[1] == None

    def test_exists_many_with_proper_parameters_without_connection(self):

        config = {'hosts': [('127.0.0.1', 3000)]}
        client1 = aerospike.client(config)

        try:
            records = client1.exists_many( self.keys, { 'timeout': 20 } )

        except ClusterError as exception:
            assert exception.code == 11L
            assert exception.msg == 'No connection to aerospike cluster'



# -*- coding: utf-8 -*-

import pytest
import sys
import time
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)


class TestUdfRemove(TestBaseClass):
    def setup_class(cls):
        """
        Setup class
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}

        if user == None and password == None:
            TestUdfRemove.client = aerospike.client(config).connect()
        else:
            TestUdfRemove.client = aerospike.client(config).connect(user, password)

    def teardown_class(cls):
        """
        Teardown class
        """

        TestUdfRemove.client.close()

    def setup_method(self, method):
        """
        Setup method
        """
        TestUdfRemove.client.udf_put(u'example.lua', 0, {})
        time.sleep(2)

    def teardown_method(self, method):
        """
        Teardown method
        """
        time.sleep(2)
        udf_list = TestUdfRemove.client.udf_list({'timeout': 100})
        for udf in udf_list:
            if udf['name'] == 'example.lua':
                TestUdfRemove.client.udf_remove("example.lua", {})

    def test_udf_remove_without_parameters(self):

        with pytest.raises(TypeError) as typeError:
            status = TestUdfRemove.client.udf_remove()
        assert "Required argument 'filename' (pos 1) not found" in typeError.value

    def test_udf_remove_with_none_as_parameters(self):

        try:
            status = TestUdfRemove.client.udf_remove(None, None)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Filename should be a string"

    def test_udf_remove_with_proper_parameters(self):

        policy = {'timeout': 50}
        module = "example.lua"
        status = TestUdfRemove.client.udf_remove(module, policy)

        assert status == 0

        time.sleep(4)
        udf_list = TestUdfRemove.client.udf_list({'timeout': 100})

        present = False
        for udf in udf_list:
            if 'example.lua' == udf['name']:
                present = True

        assert False if present else True

    def test_udf_remove_with_invalid_timeout_policy_value(self):

        policy = {'timeout': 0.1}
        module = "example.lua"

        status = TestUdfRemove.client.udf_remove(module, policy)

        assert status == 0

    def test_udf_remove_with_proper_timeout_policy_value(self):

        policy = {'timeout': 1000}
        module = "example.lua"

        status = TestUdfRemove.client.udf_remove(module, policy)

        assert status == 0
        time.sleep(3)

        udf_list = TestUdfRemove.client.udf_list({'timeout': 0})

        present = False
        for udf in udf_list:
            if 'example.lua' == udf['name']:
                present = True

        assert False if present else True

    def test_udf_remove_with_non_existent_module(self):

        policy = {}
        module = "some_module"

        try:
            status = TestUdfRemove.client.udf_remove( module, policy )

        except UDFError as exception:
            assert exception.code == 100
            assert exception.msg == "error=file_not_found\n"
            assert exception.module == "some_module"

    def test_udf_remove_with_unicode_filename(self):

        policy = {'timeout': 100}
        module = u"example.lua"
        status = TestUdfRemove.client.udf_remove(module, policy)

        assert status == 0

        time.sleep(4)
        udf_list = TestUdfRemove.client.udf_list({'timeout': 100})

        present = False
        for udf in udf_list:
            if 'example.lua' == udf['name']:
                present = True

        assert False if present else True

    def test_udf_remove_with_proper_parameters_without_connection(self):

        config = {'hosts': [('127.0.0.1', 3000)]}

        client1 = aerospike.client(config)
        policy = {'timeout': 100}
        module = "example.lua"

        try:
            status = client1.udf_remove( module, policy )

        except ClusterError as exception:
            assert exception.code == 11L
            assert exception.msg == 'No connection to aerospike cluster'



# -*- coding: utf-8 -*-

import pytest
import sys
import cPickle as pickle
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class SomeClass(object):

    pass


class TestGet(TestBaseClass):
    def setup_class(cls):
        """
        Setup method.
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            TestGet.client = aerospike.client(config).connect()
        else:
            TestGet.client = aerospike.client(config).connect(user, password)

    def teardown_class(cls):
        TestGet.client.close()

    def setup_method(self, method):

        self.keys = []
        for i in xrange(5):
            key = ('test', 'demo', i)
            rec = {'name': 'name%s' % (str(i)), 'age': i}
            TestGet.client.put(key, rec)
            self.keys.append(key)

        key = ('test', 'demo', 'list_key')
        self.keys.append(key)

        rec = {'names': ['John', 'Marlen', 'Steve']}

        TestGet.client.put(key, rec)

        key = ('test', 'demo', 'map_key')
        self.keys.append(key)

        rec = {'names': {'name': 'John', 'age': 24}}

        TestGet.client.put(key, rec)

        key = ('test', 'demo', 'list_map_key')
        self.keys.append(key)

        rec = {
            'names': ['John', 'Marlen', 'Steve'],
            'names_and_age': [{'name': 'John',
                               'age': 24}, {'name': 'Marlen',
                                            'age': 25}]
        }

        TestGet.client.put(key, rec)

        obj1, obj2 = SomeClass(), SomeClass()

        key = ('test', 'demo', 'objects')
        self.keys.append(key)

        rec = {'objects': [pickle.dumps(obj1), pickle.dumps(obj2)]}
        TestGet.client.put(key, rec)

        key = ('test', 'demo', 'bytes_key')
        self.keys.append(key)

        rec = {'bytes': bytearray('John', 'utf-8')}
        TestGet.client.put(key, rec)

        key = ('test', 'demo', 'boolean_key')
        self.keys.append(key)

        rec = {'is_present': True}
        TestGet.client.put(key, rec)

    def teardown_method(self, method):
        """
        Teardoen method.
        """
        for key in self.keys:
            TestGet.client.remove(key)

    def test_get_with_no_parameter(self):
        """
            Invoke get() without any mandatory parameters.
        """
        with pytest.raises(TypeError) as typeError:
            TestGet.client.get()

        assert "Required argument 'key' (pos 1) not found" in typeError.value

    def test_get_with_only_key(self):
        """
            Invoke get() with a key and not policy's dict.
        """
        key = ('test', 'demo', 1)

        key, meta, bins = TestGet.client.get(key)

        assert bins == {'age': 1, 'name': 'name1'}

    def test_get_with_typed_key(self):
        """
            Invoke get() with a string key and not policy's dict.
        """
        key = ('test', 'demo', '1iaushdf89v')

        try:
            key, meta, bins = TestGet.client.get( key )
            """
            We are making the api backward compatible. In case of RecordNotFound an
            exception will not be raised. Instead Ok response is returned withe the
            meta as None. This might change with further releases.
            """
        except RecordNotFound as exception:
            assert True == False
            assert exception.code == 2
            assert exception.msg == 'AEROSPIKE_ERR_RECORD_NOT_FOUND'

        assert bins == None
        assert meta == None

    def test_get_with_none_set(self):
        """
            Invoke get() with None set in key tuple.
        """
        key = ('test', None, 2)

        try:
            key, meta, bins = TestGet.client.get( key )

            """
            We are making the api backward compatible. In case of RecordNotFound an
            exception will not be raised. Instead Ok response is returned withe the
            meta as None. This might change with further releases.
            """
        except RecordNotFound as exception:
            assert True == False
            assert exception.code == 2
            assert exception.msg == 'AEROSPIKE_ERR_RECORD_NOT_FOUND'

        assert bins == None
        assert meta == None

    def test_get_with_none_namespace(self):
        """
            Invoke get() with None namespace in key tuple.
        """
        key = (None, 'demo', 2)

        try:
            key, meta, bins = TestGet.client.get( key )

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'namespace must be a string'

    def test_get_with_none_pk(self):
        """
            Invoke get() with None as primary_key part of key tuple.
        """
        key = ('test', 'demo', None)

        try:
            key, meta, bins = TestGet.client.get( key )

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'either key or digest is required'

    def test_get_with_none_key(self):
        """
            Invoke get() with None as a key.
        """
        try:
            key, meta, bins = TestGet.client.get(None)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "key is invalid"

    def test_get_key_type_list(self):
        """
            Invoke get() with key specified as a list of ns, set and key/digest.
        """
        key = ['test', 'demo', '1']

        try:
            key, meta, bins = TestGet.client.get(key)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "key is invalid"

    def test_get_with_non_existent_namespace(self):
        """
            Invoke get() for non-existent namespace.
        """
        key = ('namespace', 'demo', 1)

        try:
            key, meta, bins = TestGet.client.get(key)

        except NamespaceNotFound as exception:
            assert exception.code == 20
            assert exception.msg == 'AEROSPIKE_ERR_NAMESPACE_NOT_FOUND'

    def test_get_with_non_existent_set(self):
        """
            Invoke get() for non-existent set.
        """
        key = ('test', 'some_random_set', 1)

        try:
            key, meta, bins = TestGet.client.get(key)

            """
            We are making the api backward compatible. In case of RecordNotFound an
            exception will not be raised. Instead Ok response is returned withe the
            meta as None. This might change with further releases.
            """
        except RecordNotFound as exception:
            assert True == False
            assert exception.code == 2
            assert exception.msg == 'AEROSPIKE_ERR_RECORD_NOT_FOUND'

        assert bins == None
        assert meta == None

    def test_get_with_non_existent_key(self):
        """
            Invoke get() for non-existent key.
        """
        key = ('test', 'demo', 'non-existent')

        try:
            key, meta, bins = TestGet.client.get( key )

            """
            We are making the api backward compatible. In case of RecordNotFound an
            exception will not be raised. Instead Ok response is returned withe the
            meta as None. This might change with further releases.
            """
        except RecordNotFound as exception:
            assert True == False
            assert exception.code == 2
            assert exception.msg == 'AEROSPIKE_ERR_RECORD_NOT_FOUND'

        assert bins == None
        assert meta == None

    def test_get_for_list_type_record(self):
        """
            Invoke get() for list typed record.
        """
        key = ('test', 'demo', 'list_key')

        key, meta, bins = TestGet.client.get(key)

        assert {'names': ['John', 'Marlen', 'Steve']} == bins

    def test_get_for_map_type_record(self):
        """
            Invoke get() for map type record.
        """
        key = ('test', 'demo', 'map_key')

        key, meta, bins = TestGet.client.get(key)

        assert bins == {'names': {'name': 'John', 'age': 24}}

    def test_get_for_list_and_map_type_combined(self):
        """
            Invoke get() for list and map combined record.
        """
        key = ('test', 'demo', 'list_map_key')

        key, meta, bins = TestGet.client.get(key)

        assert bins == {
            'names': ['John', 'Marlen', 'Steve'],
            'names_and_age': [{'age': 24,
                               'name': 'John'}, {'age': 25,
                                                 'name': 'Marlen'}]
        }

    def test_get_with_list_of_objects(self):
        """
            Invoke get() for list of objects.
        """
        key = ('test', 'demo', 'objects')

        key, meta, bins = TestGet.client.get(key)

        assert bins != None

    def test_get_with_bytearray(self):
        """
            Invoke get() for bytarray record.
        """
        key = ('test', 'demo', 'bytes_key')

        key, meta, bins = TestGet.client.get(key)

        assert bins == {"bytes": bytearray('John', 'utf-8')}

    def test_get_with_key_digest(self):
        """
            Invoke get() with a key digest.
        """
        key = ('test', 'demo', 1)

        key, meta = TestGet.client.exists(key)

        key, meta, bins = TestGet.client.get((key[0], key[1], None, key[3]))

        assert bins == {"name": "name1", "age": 1}

    def test_get_for_boolean_data(self):
        """
            Invoke get() for a record having boolean data.
        """
        key = ('test', 'demo', 'boolean_key')

        _, _, bins = TestGet.client.get(key)

        assert bins == {'is_present': True}

    def test_get_initkey_with_digest(self):
        """
            Invoke get() for a record having string data.
        """
        key = ('test', 'demo', None, bytearray("asd;as[d'as;djk;uyfl",
                                               "utf-8"))
        rec = {'name': 'john'}

        policy = {'key': aerospike.POLICY_KEY_DIGEST}

        status = TestGet.client.put(key, rec, policy)

        key, meta, bins = TestGet.client.get(key, policy)

        assert bins == {'name': 'john'}
        assert key == ('test', 'demo', None,
                       bytearray(b"asd;as[d\'as;djk;uyfl"))
        TestGet.client.remove(key)

    def test_get_with_policy_key_digest(self):
        """
            Invoke get() for a record with POLICY_KEY_DIGEST
        """
        key = ('test', 'demo', 3)

        (key, _, _) = TestGet.client.get(
            key,
            policy={'key': aerospike.POLICY_KEY_DIGEST})

        assert key[2] == None

        key = ('test', 'demo', 3)

        (key, _, _) = TestGet.client.get(
            key,
            policy={'key': aerospike.POLICY_KEY_SEND})

        assert key[2] == 3

    def test_get_initkey_with_policy_send(self):
        """
            Invoke get() for a record having string data.
        """

        key = ('test', 'demo', 1)

        rec = {'name': 'john', 'age': 1}

        policy = {
            'key': aerospike.POLICY_KEY_SEND,
            'replica': aerospike.POLICY_REPLICA_ANY,
            'consistency': aerospike.POLICY_CONSISTENCY_ONE
        }

        status = TestGet.client.put(key, rec, policy)

        key, meta, bins = TestGet.client.get(key, policy)

        assert bins == {'name': 'john', 'age': 1}
        assert key == ('test', 'demo', 1, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_get_with_only_key_no_connection(self):
        """
            Invoke get() with a key and not policy's dict no connection
        """
        key = ('test', 'demo', 1)
        config = {'hosts': [('127.0.0.1', 3000)]}
        client1 = aerospike.client(config)

        try:
            key, meta, bins = client1.get( key )

        except ClusterError as exception:
            assert exception.code == 11L
            assert exception.msg == 'No connection to aerospike cluster'
